@@ -15,7 +15,7 @@ class Setup:
     STATEMENT_PATH = "broker_statements"
     TEMPLATE_PATH = "templates"
     UPDATE_PREFIX = 'jal_delta_'
-    TARGET_SCHEMA = 31
+    TARGET_SCHEMA = 32
     CALC_TOLERANCE = 1e-10
     DISP_TOLERANCE = 1e-4
 
@@ -29,41 +29,6 @@ class BookAccount:  # PREDEFINED BOOK ACCOUNTS
     Transfers = 6
 
 
-class TransactionType:   # PREDEFINED TRANSACTION TYPES - these are aligned with tabs in main window
-    NA = 0
-    Action = 1
-    Dividend = 2
-    Trade = 3
-    Transfer = 4
-    CorporateAction = 5
-
-
-class ActionSubtype:
-    SingleIncome = 1
-    SingleSpending = -1
-
-
-class TransferSubtype:   # TRANSFER SUB-TYPES
-    Fee = 0
-    Outgoing = -1
-    Incoming = 1
-
-
-class DividendSubtype:   # Subtype to distinguish dividend, bond interest etc.
-    Fee = 0
-    Dividend = 1
-    BondInterest = 2
-    StockDividend = 3
-
-
-class CorporateAction:   # CORPORATE ACTIONS FOR ASSETS
-    NA = 0
-    Merger = 1
-    SpinOff = 2
-    SymbolChange = 3
-    Split = 4
-
-
 class PredefindedAccountType:
     Cash = 1
     Bank = 2
@@ -86,7 +51,7 @@ class PredefinedCategory:
     Profit = 9
 
 
-class PredefinedAsset:
+class PredefinedAsset:   # Aligned with 'asset_types' table
     Money = 1
     Stock = 2
     Bond = 3
@@ -94,6 +59,13 @@ class PredefinedAsset:
     Commodity = 5
     Derivative = 6
     Forex = 7
+    Fund = 8
+
+
+class AssetData:
+    RegistrationCode = 1
+    ExpiryDate = 2
+    PrincipalValue = 3
 
 
 class PredefinedPeer:
@@ -107,6 +79,8 @@ class MarketDataFeed:
     US = 2
     EU = 3
     CA = 4
+    GB = 5
+    FRA = 6
 
 
 class CustomColor:
