@@ -4,10 +4,11 @@ from datetime import datetime
 from itertools import groupby
 
 from PySide6.QtWidgets import QApplication
-from jal.constants import PredefinedCategory, DividendSubtype
+from jal.constants import PredefinedCategory
 from jal.widgets.helpers import ManipulateDate
 from jal.db.db import JalDB
 from jal.db.helpers import executeSQL, readSQLrecord
+from jal.db.operations import Dividend
 from jal.data_import.statement import FOF, Statement_ImportError
 from jal.data_import.statement_xml import StatementXML
 
@@ -57,6 +58,7 @@ class IBKR_CorpActionType:
     NotSupported = -1
     _corporate_action_types = {
         'BM': FOF.ACTION_BOND_MATURITY,    # No separate value as will be converted to ordinary bond sell operation
+        'DW': FOF.ACTION_DELISTING,        # Delisting with loss of value
         'FS': FOF.ACTION_SPLIT,            # Forward split
         'HI': FOF.PAYMENT_STOCK_DIVIDEND,  # Choice dividend
         'IC': FOF.ACTION_SYMBOL_CHANGE,    # Issue change
@@ -77,79 +79,12 @@ class IBKR_CorpActionType:
 
 # -----------------------------------------------------------------------------------------------------------------------
 class IBKR_Currency:
-    def __init__(self, assets_list, code):
-        self.id = None
-        match = [x for x in assets_list if x['symbol'] == code and x['type'] == FOF.ASSET_MONEY]
-        if match:
-            if len(match) == 1:
-                self.id = match[0]["id"]
-            else:
-                logging.error(QApplication.translate("IBKR", "Multiple match for ") + f"{code}")
-        else:
-            self.id = max([0] + [x['id'] for x in assets_list]) + 1
-            currency = {"id": self.id, "type": "money", "symbol": code}
-            assets_list.append(currency)
-
+    pass
 
 # -----------------------------------------------------------------------------------------------------------------------
 class IBKR_Asset:
     BondPrincipal = 1000
 
-    def __init__(self, assets_list, symbol, category, name, isin, cusip, exchange=''):
-        self.id = None
-        self.assets = assets_list
-
-        if symbol.endswith('.OLD'):
-            symbol = symbol[:-len('.OLD')]
-
-        if self.match_and_update('isin', isin, {'symbol': symbol, 'reg_code': cusip}):
-            return
-        if self.match_and_update('reg_code', cusip, {'symbol': symbol, 'isin': isin}):
-            return
-        if self.match_and_update('symbol', symbol, {'isin': isin, 'reg_code': cusip}):
-            return
-        if category == IBKR_AssetType.NotSupported:
-            if symbol:
-                logging.warning(self.tr("Asset type isn't supported: ") + f"'{category}' ({symbol})")
-            return
-        self.id = max([0] + [x['id'] for x in assets_list]) + 1
-        asset = {"id": self.id, "symbol": symbol, 'name': name, 'type': category}
-        if isin:
-            asset['isin'] = isin
-        if cusip:
-            asset['reg_code'] = cusip
-        if exchange and exchange != "VALUE":   # store exchange only if it is valuable
-            asset['exchange'] = exchange
-        assets_list.append(asset)
-
-    def tr(self, text):
-        return QApplication.translate("IBKR", text)
-
-    # search in self.assets['match_key'] for value match_value
-    # iterate through key:value pairs of updates to update self.assets['key'] with 'value'
-    # assign self.id if asset was found and returns True, otherwise returns False
-    def match_and_update(self, match_key, match_value, updates):
-        if not match_value:
-            return False
-        try:
-            match = [x for x in self.assets if match_key in x and x[match_key] == match_value]
-        except KeyError:
-            match = []
-        if match:
-            if len(match) == 1:
-                asset = match[0]
-                for key in updates:
-                    if updates[key]:
-                        if (key == 'symbol') and (asset[key] + 'D' == updates[key] or asset[key] + 'Q' == updates[key]):
-                            continue  # Don't update symbols due to known bankruptcy or new issue patterns
-                        if asset[key] != updates[key]:
-                            asset[key] = updates[key]
-                self.id = asset['id']
-                return True
-            else:
-                logging.error(self.tr("Multiple asset match for ") + f"'{match_key}':'{match_value}', {updates}")
-                return False
-
 
 # -----------------------------------------------------------------------------------------------------------------------
 class IBKR_Account:
@@ -215,10 +150,11 @@ class StatementIBKR(StatementXML):
             'SecuritiesInfo': {'tag': 'SecurityInfo',
                                'level': '',
                                'values': [('symbol', 'symbol', str, None),
+                                          ('currency', 'currency', IBKR_Currency, None),
                                           ('assetCategory', 'type', IBKR_AssetType, IBKR_AssetType.NotSupported),
                                           ('description', 'name', str, None),
                                           ('isin', 'isin', str, ''),
-                                          ('cusip', 'reg_code', str, ''),
+                                          ('cusip', 'reg_number', str, ''),
                                           ('expiry', 'expiry', datetime, 0),
                                           ('maturity', 'maturity', datetime, 0),
                                           ('listingExchange', 'exchange', str, '')],
@@ -257,8 +193,7 @@ class StatementIBKR(StatementXML):
                                  'values': [('type', 'type', IBKR_CorpActionType, IBKR_CorpActionType.NotSupported),
                                             ('accountId', 'account', IBKR_Account, None),
                                             ('symbol', 'asset', IBKR_Asset, None),
-                                            (
-                                            'assetCategory', 'asset_type', IBKR_AssetType, IBKR_AssetType.NotSupported),
+                                            ('assetCategory', 'asset_type', IBKR_AssetType, IBKR_AssetType.NotSupported),
                                             ('dateTime', 'timestamp', datetime, None),
                                             ('transactionID', 'number', str, ''),
                                             ('description', 'description', str, None),
@@ -313,15 +248,15 @@ class StatementIBKR(StatementXML):
         if attr_name not in xml_element.attrib:
             return default_value
         asset_category = self.attr_asset_type(xml_element, 'assetCategory', None)
-        if asset_category != FOF.ASSET_STOCK and asset_category != FOF.ASSET_BOND:
-            logging.error(self.tr("Corporate actions are supported for stocks or bonds only"))
+        if asset_category not in [FOF.ASSET_STOCK, FOF.ASSET_BOND, FOF.ASSET_WARRANT]:
+            logging.error(self.tr("Corporate action isn't supported for asset type: ") + f"'{asset_category}'")
             return default_value
         return IBKR_CorpActionType(xml_element.attrib[attr_name]).type
 
     def attr_currency(self, xml_element, attr_name, default_value):
         if attr_name not in xml_element.attrib:
             return default_value
-        currency_id = IBKR_Currency(self._data[FOF.ASSETS], xml_element.attrib[attr_name]).id
+        currency_id = self.currency_id(xml_element.attrib[attr_name])
         if currency_id is None:
             return default_value
         else:
@@ -330,23 +265,36 @@ class StatementIBKR(StatementXML):
     def attr_asset(self, xml_element, attr_name, default_value):
         if attr_name not in xml_element.attrib:
             return default_value
-        asset_category = self.attr_asset_type(xml_element, 'assetCategory', None)
+        if xml_element.attrib[attr_name] == '':
+            return default_value
+        if xml_element.tag == 'CashTransaction' and attr_name == 'currency':
+            asset_category = FOF.ASSET_MONEY
+        else:
+            asset_category = self.attr_asset_type(xml_element, 'assetCategory', None)
         if xml_element.tag == 'Trade' and asset_category == FOF.ASSET_MONEY:
             currency = xml_element.attrib[attr_name].split('.')
-            asset_id = [IBKR_Currency(self._data[FOF.ASSETS], code).id for code in currency]
+            asset_id = [self.currency_id(code) for code in currency]
             if not asset_id:
                 return default_value
         else:
-            name = ''
-            if xml_element.tag not in ['CorporateAction', 'CashTransaction']:
-                name = xml_element.attrib['description'] if 'description' in xml_element.attrib else ''
-            isin = xml_element.attrib['isin'] if 'isin' in xml_element.attrib else ''
-            cusip = xml_element.attrib['cusip'] if 'cusip' in xml_element.attrib else ''
-            exchange = xml_element.attrib['listingExchange'] if 'listingExchange' in xml_element.attrib else ''
-            asset_id = IBKR_Asset(self._data[FOF.ASSETS], xml_element.attrib[attr_name], asset_category,
-                                  name, isin, cusip, exchange).id
-            if asset_id is None:
-                return default_value
+            symbol = xml_element.attrib[attr_name]
+            if symbol.endswith('.OLD'):
+                symbol = symbol[:-len('.OLD')]
+            if asset_category == IBKR_AssetType.NotSupported:
+                raise Statement_ImportError(self.tr("Asset type isn't supported: ") + f"'{asset_category}' ({symbol})")
+            asset_data = {'symbol': symbol, 'type': asset_category}
+            if xml_element.tag not in ['CorporateAction', 'CashTransaction'] and 'description' in xml_element.attrib:
+                asset_data['name'] = xml_element.attrib['description']
+            if asset_category != FOF.ASSET_MONEY:
+                asset_data['currency'] = self.currency_id(xml_element.attrib['currency'])
+            if 'isin' in xml_element.attrib and xml_element.attrib['isin']:
+                asset_data['isin'] = xml_element.attrib['isin']
+            if 'cusip' in xml_element.attrib and xml_element.attrib['cusip']:
+                asset_data['reg_number'] = xml_element.attrib['cusip']
+            if 'listingExchange' in xml_element.attrib and xml_element.attrib['listingExchange'] \
+                    and xml_element.attrib['listingExchange'] != 'VALUE':  # don't store 'VALUE' or empty exchange
+                asset_data['note'] = xml_element.attrib['listingExchange']
+            asset_id = self.asset_id(asset_data)
         return asset_id
 
     def attr_account(self, xml_element, attr_name, default_value):
@@ -365,7 +313,7 @@ class StatementIBKR(StatementXML):
                     logging.error(self.tr("Can't get account currency for account: ") + f"{xml_element}")
                 return default_value
             currency = [xml_element.attrib['currency']]
-        currency_ids = [IBKR_Currency(self._data[FOF.ASSETS], code).id for code in currency]
+        currency_ids = [self.currency_id(code) for code in currency]
         account_id = IBKR_Account(self._data[FOF.ACCOUNTS], xml_element.attrib[attr_name], currency_ids).id
         if account_id is None:
             return default_value
@@ -376,9 +324,9 @@ class StatementIBKR(StatementXML):
         candidates = [x for x in self._data[FOF.ASSETS] if 'isin' in x and x['isin'] == isin]
         if len(candidates) == 1:
             return candidates[0]["id"]
-        candidates = [x for x in self._data[FOF.ASSETS] if 'symbol' in x and x['symbol'] == symbol]
+        candidates = [x for x in self._data[FOF.SYMBOLS] if 'symbol' in x and x['symbol'] == symbol]
         if len(candidates) == 1:
-            return candidates[0]["id"]
+            return candidates[0]["asset"]
         return 0
 
     def set_asset_counry(self, asset_id, country):
@@ -401,29 +349,21 @@ class StatementIBKR(StatementXML):
 
     def load_assets(self, assets):
         asset_count = 0
-        stored = set()
-        base = max([0] + [x['id'] for x in self._data[FOF.ASSETS]]) + 1
         for asset in assets:
             if asset['type'] == IBKR_AssetType.NotSupported:   # Skip not supported type of asset
                 continue
-
-            asset_data = tuple(asset.items())
-            if asset_data in stored:  # Skip duplicated assets
-                continue
-            stored.add(asset_data)
-
-            asset['id'] = base + asset_count
             # IB may use '.OLD' suffix if asset is being replaced
             asset['symbol'] = asset['symbol'][:-len('.OLD')] if asset['symbol'].endswith('.OLD') else asset['symbol']
-            if asset['exchange'] == '' or asset['exchange'] == 'VALUE':  # don't store 'VALUE' or empty exchange
-                asset.pop('exchange')
+            if asset['exchange'] and asset['exchange'] != 'VALUE':  # don't store 'VALUE' or empty exchange
+                asset['note'] = asset['exchange']
             if asset['maturity']:
                 asset['expiry'] = asset['maturity']
             if asset['expiry'] == 0:
                 asset.pop('expiry')
             asset.pop('maturity')
+            asset.pop('exchange')
+            self.asset_id(asset)
             asset_count += 1
-            self._data[FOF.ASSETS].append(asset)
         logging.info(self.tr("Securities loaded: ") + f"{asset_count} ({len(assets)})")
 
     def load_ib_trades(self, ib_trades):
@@ -506,7 +446,8 @@ class StatementIBKR(StatementXML):
             FOF.ACTION_SYMBOL_CHANGE: self.load_symbol_change,
             FOF.PAYMENT_STOCK_DIVIDEND: self.load_stock_dividend,
             FOF.ACTION_SPLIT: self.load_split,
-            FOF.ACTION_BOND_MATURITY: self.load_bond_maturity
+            FOF.ACTION_BOND_MATURITY: self.load_bond_maturity,
+            FOF.ACTION_DELISTING: self.load_delisting
         }
 
         cnt = 0
@@ -535,7 +476,7 @@ class StatementIBKR(StatementXML):
                 cnt += action_loaders[action['type']](action, parts_b)
             else:
                 raise Statement_ImportError(
-                    self.tr("Corporate action type is not supported: ") + f"{action['type']}")
+                    self.tr("Corporate action type is not supported: ") + f"{action}")
         logging.info(self.tr("Corporate actions loaded: ") + f"{cnt} ({len(actions)})")
 
     # Find record in list 'parts_b' (second parts of corporate actions) which matches
@@ -570,10 +511,11 @@ class StatementIBKR(StatementXML):
 
     def load_merger(self, action, parts_b) -> int:
         MergerPatterns = [
-            r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +MERGED\(\w+\) +WITH +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
-            r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\(\w+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
-            r"^(?P<symbol_old>\w+)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\(\w+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+), +(?P<isin_new2>\w+) +(?P<X2>\d+) +FOR +(?P<Y2>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
-            r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +TENDERED TO +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), +(?P<name>.*), +(?P<id>.*)\)$"
+            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +MERGED\([\w ]+\) +WITH +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\([\w ]+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +AND +(?P<currency>.*) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>.*)(.OLD)?\((?P<isin_old>\w+)\) +CASH and STOCK MERGER +\([\w ]+\) +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+), +(?P<isin_new2>.*) +(?P<X2>\d+) +FOR +(?P<Y2>\d+) +AND +(?P<currency>\w+) +(\d+(\.\d+)?) +\((?P<symbol>\w+)(.OLD)?, (?P<name>.*), (?P<id>\w+)\)$",
+            r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +TENDERED TO +(?P<isin_new>\w+) +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), +(?P<name>.*), +(?P<id>.*)\)$",
+            r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +MERGED\([\w ]+\) +FOR (?P<currency>\w+) (?P<price>\d+\.\d+) PER SHARE +\((?P<symbol>.*), (?P<name>.*)( - TENDER ODD LOT)?, (?P<id>\w+)\)$"  # "TENDER ODD LOT" part is optional
             ]
 
         parts = None
@@ -582,13 +524,27 @@ class StatementIBKR(StatementXML):
             parts = re.match(pattern, action['description'], re.IGNORECASE)
             if parts:
                 break
+
         if parts is None:
             raise Statement_ImportError(self.tr("Can't parse Merger description ") + f"'{action}'")
         merger_a = parts.groupdict()
+
         if len(merger_a) != MergerPatterns[pattern_id].count("(?P<"):  # check expected number of matches
             raise Statement_ImportError(self.tr("Merger description miss some data ") + f"'{action}'")
+
         description_b = action['description'][:parts.span('symbol')[0]] + merger_a['symbol_old']
         asset_b = self.locate_asset(merger_a['symbol_old'], merger_a['isin_old'])
+
+        if pattern_id == 4:  # Asset converted to money -> we store it as a sell trade
+            action['id'] = max([0] + [x['id'] for x in self._data[FOF.TRADES]]) + 1
+            action['settlement'] = action['timestamp']
+            action['price'] = action['proceeds'] / (-action['quantity'])
+            action['note'] = action.pop('description')
+            action['fee'] = 0.0
+            self.drop_extra_fields(action, ["type", "value", "proceeds", "code", "asset_type", "jal_processed"])
+            self._data[FOF.TRADES].append(action)
+            return 1
+
         paired_record = self.find_corp_action_pair(asset_b, description_b, action, parts_b)
         if (pattern_id == 1 or pattern_id == 2) and (not paired_record[0]['jal_processed']):
             self.add_merger_payment(action['timestamp'], action['account'], paired_record[0]['proceeds'],
@@ -606,11 +562,13 @@ class StatementIBKR(StatementXML):
             action['quantity'] = [-paired_record[0]['quantity'], action['quantity']]
         self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
         self._data[FOF.CORP_ACTIONS].append(action)
+
         paired_record[0]['jal_processed'] = True
+
         return 2
 
     def add_merger_payment(self, timestamp, account_id, amount, currency, description):
-        currency_id = IBKR_Currency(self._data[FOF.ASSETS], currency).id
+        currency_id = self.currency_id(currency)
         account = [x for x in self._data[FOF.ACCOUNTS] if x['id'] == account_id][0]
         if account['currency'] != currency_id:
             account_id = IBKR_Account(self._data[FOF.ACCOUNTS], account['number'], [currency_id]).id
@@ -620,7 +578,7 @@ class StatementIBKR(StatementXML):
         self._data[FOF.INCOME_SPENDING].append(payment)
 
     def load_spinoff(self, action, _parts_b) -> int:
-        SpinOffPattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +SPINOFF +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+        SpinOffPattern = r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +SPINOFF +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), (?P<name>.*), (?P<id>\w+)\)$"
 
         parts = re.match(SpinOffPattern, action['description'], re.IGNORECASE)
         if parts is None:
@@ -644,7 +602,7 @@ class StatementIBKR(StatementXML):
         return 1
 
     def load_symbol_change(self, action, parts_b) -> int:
-        SymbolChangePattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +CUSIP\/ISIN CHANGE TO +\((?P<isin_new>\w+)\) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+        SymbolChangePattern = r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +CUSIP\/ISIN CHANGE TO +\((?P<isin_new>\w+)\) +\((?P<symbol>.*), (?P<name>.*), (?P<id>\w+)\)$"
 
         parts = re.match(SymbolChangePattern, action['description'], re.IGNORECASE)
         if parts is None:
@@ -681,7 +639,7 @@ class StatementIBKR(StatementXML):
         return 1
 
     def load_split(self, action, parts_b) -> int:
-        SplitPattern = r"^(?P<symbol_old>\w+)\((?P<isin_old>\w+)\) +SPLIT +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>\w+), (?P<name>.*), (?P<id>\w+)\)$"
+        SplitPattern = r"^(?P<symbol_old>.*)\((?P<isin_old>\w+)\) +SPLIT +(?P<X>\d+) +FOR +(?P<Y>\d+) +\((?P<symbol>.*), (?P<name>.*), (?P<id>\w+)\)$"
 
         parts = re.match(SplitPattern, action['description'], re.IGNORECASE)
         if parts is None:
@@ -726,6 +684,15 @@ class StatementIBKR(StatementXML):
         self._data[FOF.TRADES].append(action)
         return 1
 
+    def load_delisting(self, action, parts_b) -> int:
+        action['id'] = max([0] + [x['id'] for x in self._data[FOF.CORP_ACTIONS]]) + 1
+        action['asset'] = [action['asset'], action['asset']]
+        action['quantity'] = [-action['quantity'], 0.0]
+        action['cost_basis'] = 1.0
+        self.drop_extra_fields(action, ["value", "proceeds", "code", "asset_type", "jal_processed"])
+        self._data[FOF.CORP_ACTIONS].append(action)
+        return 1
+
     def load_cash_transactions(self, cash):
         cnt = 0
 
@@ -842,16 +809,18 @@ class StatementIBKR(StatementXML):
                      and x['asset'] == asset_id and x['account'] == account_id]
         account = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]
         currency = [x for x in self._data[FOF.ASSETS] if x["id"] == account['currency']][0]
-        db_account = JalDB().get_account_id(account['number'], currency['symbol'])
+        currency_symbol = [x for x in self._data[FOF.SYMBOLS] if x["asset"] == currency['id']][0]
+        db_account = JalDB().get_account_id(account['number'], currency_symbol['symbol'])
         asset = [x for x in self._data[FOF.ASSETS] if x["id"] == asset_id][0]
         isin = asset['isin'] if 'isin' in asset else ''
-        db_asset = JalDB().get_asset_id(asset['symbol'], isin=isin, dialog_new=False)
+        symbols = [x for x in self._data[FOF.SYMBOLS] if x["asset"] == asset_id]
+        db_asset = JalDB().get_asset_id({'isin': isin, 'symbol': symbols[0]['symbol']})
         if db_account is not None and db_asset is not None:
             query = executeSQL(
                 "SELECT -id AS id, -account_id AS account, timestamp, number, "
                 "-asset_id AS asset, amount, tax, note as description FROM dividends "
                 "WHERE type=:div AND account_id=:account_id AND asset_id=:asset_id",
-                [(":div", DividendSubtype.Dividend), (":account_id", db_account), (":asset_id", db_asset)],
+                [(":div", Dividend.Dividend), (":account_id", db_account), (":asset_id", db_asset)],
                 forward_only=True)
             while query.next():
                 db_dividend = readSQLrecord(query, named=True)
