@@ -1,26 +1,23 @@
 import io
-import re
 import time
 import json
 import logging
 import pandas as pd
 from urllib.parse import parse_qs
-from PySide2.QtWidgets import QStyledItemDelegate
+from PySide6.QtWidgets import QStyledItemDelegate
 from jal.widgets.reference_selector import CategorySelector, TagSelector
 from jal.constants import CustomColor
 from jal.db.helpers import get_category_name
 try:
     from pyzbar import pyzbar
-    from PIL import Image
+    from PIL import Image, UnidentifiedImageError
 except ImportError:
     pass   # We should not be in this module as dependencies have been checked in main_window.py and calls are disabled
 
 
-from PySide2.QtCore import Qt, Slot, Signal, QDateTime, QBuffer, QThread, QAbstractTableModel
-from PySide2.QtWidgets import QApplication, QDialog, QFileDialog, QHeaderView
-# This QCamera staff ran good on Windows but didn't fly on Linux from the box until 'cheese' installation
-from PySide2.QtMultimedia import QCameraInfo, QCamera, QCameraImageCapture, QVideoFrame
-from jal.widgets.helpers import g_tr, dependency_present
+from PySide6.QtCore import Qt, Slot, Signal, QDateTime, QBuffer, QThread, QAbstractTableModel
+from PySide6.QtWidgets import QApplication, QDialog, QFileDialog, QHeaderView
+from jal.widgets.helpers import dependency_present
 from jal.db.helpers import executeSQL, readSQL
 from jal.data_import.slips_tax import SlipsTaxAPI
 from jal.ui.ui_slip_import_dlg import Ui_ImportSlipDlg
@@ -57,17 +54,18 @@ class PandasLinesModel(QAbstractTableModel):
                 self._data.iloc[index.row(), index.column()] = value
                 self.dataChanged.emit(index, index)
                 return True
+        return False
 
     def headerData(self, col, orientation, role=Qt.DisplayRole):
-        if (orientation == Qt.Horizontal and role == Qt.DisplayRole):
+        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if col == 0:
-                return g_tr('PandasLinesModel', "Product name")
+                return self.tr("Product name")
             if col == 1:
-                return g_tr('PandasLinesModel', "Category")
+                return self.tr("Category")
             if col == 3:
-                return g_tr('PandasLinesModel', "Tag")
+                return self.tr("Tag")
             if col == 4:
-                return g_tr('PandasLinesModel', "Amount")
+                return self.tr("Amount")
         return None
 
 
@@ -122,7 +120,7 @@ class SlipLinesDelegate(QStyledItemDelegate):
 class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     qr_data_available = Signal(str)
     qr_data_validated = Signal()
-    json_data_available = Signal()
+    qr_data_loaded = Signal()
 
     OPERATION_PURCHASE = 1
     OPERATION_RETURN = 2
@@ -137,32 +135,33 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         self.delegate = []
 
         self.CameraGroup.setVisible(False)
-        self.cameraActive = False
-        self.camera = None
-        self.img_capture = None
 
         self.QR_data = ''
         self.slip_json = None
         self.slip_lines = None
 
         self.slipsAPI = SlipsTaxAPI()
+        self.tensor_flow_present = dependency_present(['tensorflow'])
 
         self.qr_data_available.connect(self.parseQRdata)
+        self.qr_data_validated.connect(self.downloadSlipJSON)
+        self.qr_data_loaded.connect(self.recognizeCategories)
         self.LoadQRfromFileBtn.clicked.connect(self.loadFileQR)
         self.GetQRfromClipboardBtn.clicked.connect(self.readClipboardQR)
         self.GetQRfromCameraBtn.clicked.connect(self.readCameraQR)
         self.StopCameraBtn.clicked.connect(self.closeCamera)
+        self.ScannerQR.decodedQR.connect(self.onCameraQR)
         self.GetSlipBtn.clicked.connect(self.downloadSlipJSON)
         self.LoadJSONfromFileBtn.clicked.connect(self.loadFileSlipJSON)
         self.AddOperationBtn.clicked.connect(self.addOperation)
         self.ClearBtn.clicked.connect(self.clearSlipData)
         self.AssignCategoryBtn.clicked.connect(self.recognizeCategories)
 
-        self.AssignCategoryBtn.setEnabled(dependency_present(['tensorflow']))
+        self.AssignCategoryBtn.setEnabled(self.tensor_flow_present)
 
     def closeEvent(self, arg__1):
-        if self.cameraActive:
-            self.closeCamera()
+        self.ScannerQR.stopScan()
+        self.accept()
 
     def initUi(self):
         self.SlipAmount.setText('')
@@ -178,7 +177,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     def loadFileQR(self):
         self.initUi()
         qr_file, _filter = \
-            QFileDialog.getOpenFileName(self, g_tr('ImportSlipDialog', "Select file with QR code"),
+            QFileDialog.getOpenFileName(self, self.tr("Select file with QR code"),
                                         ".", "JPEG images (*.jpg);;PNG images (*.png)")
         if qr_file:
             barcodes = pyzbar.decode(Image.open(qr_file), symbols=[pyzbar.ZBarSymbol.QRCODE])
@@ -193,10 +192,14 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     # Returns: True if QR found, False if no QR found
     # Emits: qr_data_available(str: qr_data) if QR found
     def readImageQR(self, image):
-        buffer = QBuffer()
+        buffer = QBuffer()   # TODO: the same code is present in qr_scanner.py -> move to one place
         buffer.open(QBuffer.ReadWrite)
         image.save(buffer, "BMP")
-        pillow_image = Image.open(io.BytesIO(buffer.data()))
+        try:
+            pillow_image = Image.open(io.BytesIO(buffer.data()))
+        except UnidentifiedImageError:
+            logging.warning(self.tr("Image format isn't supported"))
+            return False
         barcodes = pyzbar.decode(pillow_image, symbols=[pyzbar.ZBarSymbol.QRCODE])
         if barcodes:
             self.qr_data_available.emit(barcodes[0].data.decode('utf-8'))
@@ -208,70 +211,25 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     def readClipboardQR(self):
         self.initUi()
         if not self.readImageQR(QApplication.clipboard().image()):
-            logging.warning('ImportSlipDialog', "No QR codes found in clipboard")
+            logging.warning(self.tr("No QR codes found in clipboard"))
 
     @Slot()
     def readCameraQR(self):
         self.initUi()
-        if len(QCameraInfo.availableCameras()) == 0:
-            logging.warning(g_tr('ImportSlipDialog', "There are no cameras available"))
-            return
-        self.cameraActive = True
         self.CameraGroup.setVisible(True)
         self.SlipDataGroup.setVisible(False)
-
-        camera_info = QCameraInfo.defaultCamera()
-        logging.info(g_tr('ImportSlipDialog', "Read QR with camera: " + camera_info.deviceName()))
-        self.camera = QCamera(camera_info)
-        self.camera.errorOccurred.connect(self.onCameraError)
-        self.img_capture = QCameraImageCapture(self.camera)
-        self.img_capture.setCaptureDestination(QCameraImageCapture.CaptureToBuffer)
-        self.img_capture.setBufferFormat(QVideoFrame.Format_RGB32)
-        self.img_capture.error.connect(self.onCameraCaptureError)
-        self.img_capture.readyForCaptureChanged.connect(self.onReadyForCapture)
-        self.img_capture.imageAvailable.connect(self.onCameraImageReady)
-        self.camera.setViewfinder(self.Viewfinder)
-        self.camera.setCaptureMode(QCamera.CaptureStillImage)
-        self.camera.start()
+        self.ScannerQR.startScan()
 
     @Slot()
     def closeCamera(self):
-        self.camera.stop()
-        self.camera.unload()
+        self.ScannerQR.stopScan()
         self.CameraGroup.setVisible(False)
         self.SlipDataGroup.setVisible(True)
-        self.img_capture = None
-        self.camera = None
-        self.cameraActive = False
 
     @Slot()
-    def onCameraError(self, error):
-        logging.error(g_tr('ImportSlipDialog', "Camera error: " + str(error) + " / " + self.camera.errorString()))
-
-    @Slot()
-    def onCameraCaptureError(self, _id, error, msg):
-        logging.error(g_tr('ImportSlipDialog', "Camera error: " + str(error) + " / " + msg))
-
-    #----------------------------------------------------------------------
-    # This event happens once upon camera start - it triggers first capture
-    # Consequent captures will be initiated after image processing in self.onCameraImageReady
-    @Slot()
-    def onReadyForCapture(self):
-        self.camera.searchAndLock()
-        self.img_capture.capture()
-        self.camera.unlock()
-
-    #----------------------------------------------------------------------
-    # Try to decode QR from captured frame
-    # Close camera if decoded successfully otherwise try to capture again
-    def onCameraImageReady(self, _id, captured_image):
-        if self.readImageQR(captured_image.image()):
-            self.closeCamera()
-        else:
-            QThread.sleep(1)
-            self.camera.searchAndLock()
-            self.img_capture.capture()
-            self.camera.unlock()
+    def onCameraQR(self, decoded_qr):
+        self.closeCamera()
+        self.qr_data_available.emit(decoded_qr)
 
     #-----------------------------------------------------------------------------------------------
     # Check if available QR data matches with self.QR_pattern
@@ -280,11 +238,12 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     def parseQRdata(self, qr_data):
         self.QR_data = qr_data
 
-        logging.info(g_tr('ImportSlipDialog', "QR: " + self.QR_data))
+        logging.info(self.tr("QR: " + self.QR_data))
         params = parse_qs(self.QR_data)
         try:
             for timestamp_pattern in self.timestamp_patterns:
                 datetime = QDateTime.fromString(params['t'][0], timestamp_pattern)
+                datetime.setTimeSpec(Qt.UTC)
                 if datetime.isValid():
                     self.SlipTimstamp.setDateTime(datetime)
             self.SlipAmount.setText(params['s'][0])
@@ -293,7 +252,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
             self.FP.setText(params['fp'][0])
             self.SlipType.setCurrentIndex(int(params['n'][0]) - 1)
         except KeyError:
-            logging.warning(g_tr('ImportSlipDialog', "QR available but pattern isn't recognized: " + self.QR_data))
+            logging.warning(self.tr("QR available but pattern isn't recognized: " + self.QR_data))
             return
         self.qr_data_validated.emit()
 
@@ -307,7 +266,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
             if result != SlipsTaxAPI.Pending:
                 break
             if attempt > 5:
-                logging.warning(g_tr('ImportSlipDialog', "Max retry count exceeded."))
+                logging.warning(self.tr("Max retry count exceeded."))
                 break
             attempt += 1
             time.sleep(0.5) # wait half a second before next attempt
@@ -319,7 +278,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     @Slot()
     def loadFileSlipJSON(self):
         json_file, _filter = \
-            QFileDialog.getOpenFileName(self, g_tr('ImportSlipDialog', "Select file with slip JSON data"),
+            QFileDialog.getOpenFileName(self, self.tr("Select file with slip JSON data"),
                                         ".", "JSON files (*.json)")
         if json_file:
             with open(json_file) as f:
@@ -335,10 +294,10 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
                 if 'receipt' in sub:
                     slip = sub['receipt']
                 else:
-                    logging.error(g_tr('ImportSlipDialog', "Can't find 'receipt' tag in json 'document'"))
+                    logging.error(self.tr("Can't find 'receipt' tag in json 'document'"))
                     return
             else:
-                logging.error(g_tr('ImportSlipDialog', "Can't find 'document' tag in json 'ticket'"))
+                logging.error(self.tr("Can't find 'document' tag in json 'ticket'"))
                 return
         else:
             slip = self.slip_json
@@ -348,7 +307,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         if 'operationType' in slip:
             operation = int(slip['operationType'])
         else:
-            logging.error(g_tr('ImportSlipDialog', "Can't find 'operationType' tag in json 'ticket'"))
+            logging.error(self.tr("Can't find 'operationType' tag in json 'ticket'"))
             return
         # Get shop name
         shop_name = ''
@@ -369,8 +328,8 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
 
         # Get date from timestamp
         if 'dateTime' in slip:
-            slip_datetime = QDateTime()
-            slip_datetime.setSecsSinceEpoch(int(slip['dateTime']))
+            slip_datetime = QDateTime.fromSecsSinceEpoch(int(slip['dateTime']))
+            slip_datetime.setTimeSpec(Qt.UTC)
             self.SlipDateTime.setDateTime(slip_datetime)
 
         # Convert price to roubles
@@ -380,7 +339,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         elif operation == self.OPERATION_RETURN:
             self.slip_lines['sum'] = self.slip_lines['sum'] / 100
         else:
-            logging.error(g_tr('ImportSlipDialog', "Unknown operation type ") + f"{operation}")
+            logging.error(self.tr("Unknown operation type ") + f"{operation}")
             return
         # Use quantity if it differs from 1 unit value
         self.slip_lines.loc[self.slip_lines['quantity'] != 1, 'name'] = \
@@ -410,17 +369,17 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         font.setBold(True)
         self.LinesTableView.horizontalHeader().setFont(font)
         self.LinesTableView.show()
+        self.qr_data_loaded.emit()
 
     def addOperation(self):
         if self.AccountEdit.selected_id == 0:
-            logging.warning(g_tr('ImportSlipDialog', "Not possible to import slip: no account set for import"))
+            logging.warning(self.tr("Not possible to import slip: no account set for import"))
             return
         if self.PeerEdit.selected_id == 0:
-            logging.warning(g_tr('ImportSlipDialog',
-                                 "Not possible to import slip: can't import: no peer set for import"))
+            logging.warning(self.tr("Not possible to import slip: can't import: no peer set for import"))
             return
         if self.slip_lines[self.slip_lines['category'] == 0].shape[0] != 0:
-            logging.warning(g_tr('ImportSlipDialog', "Not possible to import slip: some categories are not set"))
+            logging.warning(self.tr("Not possible to import slip: some categories are not set"))
             return
 
         query = executeSQL("INSERT INTO actions (timestamp, account_id, peer_id) "
@@ -457,6 +416,8 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
 
     @Slot()
     def recognizeCategories(self):
+        if not self.tensor_flow_present:
+            return
         self.slip_lines['category'], self.slip_lines['confidence'] = \
             recognize_categories(self.slip_lines['name'].tolist())
         self.model.dataChanged.emit(None, None)  # refresh full view
\ No newline at end of file
