@@ -8,17 +8,22 @@ from datetime import datetime, timezone
 from collections import defaultdict
 
 from PySide6.QtCore import QObject
-from PySide6.QtWidgets import QApplication, QDialog, QMessageBox
-from jal.constants import Setup, MarketDataFeed, PredefinedAsset, DividendSubtype, CorporateAction
+from PySide6.QtWidgets import QDialog, QMessageBox
+from jal.constants import Setup, MarketDataFeed, PredefinedAsset
+from jal.db.settings import JalSettings
 from jal.db.helpers import account_last_date, get_app_path
 from jal.db.db import JalDB
+from jal.db.operations import Dividend, CorporateAction
 from jal.widgets.account_select import SelectAccountDialog
+from jal.net.downloader import QuoteDownloader
 
 
 class FOF:
     PERIOD = "period"
     ACCOUNTS = "accounts"
     ASSETS = "assets"
+    SYMBOLS = "symbols"
+    ASSETS_DATA = "assets_data"
     TRADES = "trades"
     TRANSFERS = "transfers"
     CORP_ACTIONS = "corporate_actions"
@@ -39,6 +44,7 @@ class FOF:
     ACTION_SPINOFF = "spin-off"
     ACTION_SYMBOL_CHANGE = "symbol_change"
     ACTION_BOND_MATURITY = "bond_maturity"    # Isn't used in reality as will be put as ordinary sell operation
+    ACTION_DELISTING = "delisting"
 
     PAYMENT_DIVIDEND = "dividend"
     PAYMENT_INTEREST = "interest"
@@ -78,7 +84,8 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         FOF.ACTION_MERGER: CorporateAction.Merger,
         FOF.ACTION_SPLIT: CorporateAction.Split,
         FOF.ACTION_SPINOFF: CorporateAction.SpinOff,
-        FOF.ACTION_SYMBOL_CHANGE: CorporateAction.SymbolChange
+        FOF.ACTION_SYMBOL_CHANGE: CorporateAction.SymbolChange,
+        FOF.ACTION_DELISTING: CorporateAction.Delisting
     }
     _sources = {
         'NYSE': MarketDataFeed.US,
@@ -98,6 +105,8 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         self._section_loaders = {
             FOF.PERIOD: self._check_period,
             FOF.ASSETS: self._import_assets,
+            FOF.SYMBOLS: self._import_symbol_tickers,
+            FOF.ASSETS_DATA: self._import_asset_data,
             FOF.ACCOUNTS: self._import_accounts,
             FOF.INCOME_SPENDING: self._import_imcomes_and_spendings,
             FOF.TRANSFERS: self._import_transfers,
@@ -133,30 +142,65 @@ class Statement(QObject):   # derived from QObject to have proper string transla
         if unsupported_sections:
             for section in unsupported_sections:
                 self._data.pop(section)
-            logging.warning(self.tr("Some sections are not supported: ") + unsupported_sections)
+            logging.warning(self.tr("Some sections are not supported: ") + f"{unsupported_sections}")
 
     # check are assets and accounts from self._data present in database
     # replace IDs in self._data with IDs from database (DB IDs will be negative, initial IDs will be positive)
-    def match_db_ids(self, verbal=True):
-        self._match_asset_ids(verbal)
+    def match_db_ids(self):
+        self._match_currencies()
+        self._match_asset_isin()
+        self._match_asset_reg_number()
+        self._match_asset_symbol()
         self._match_account_ids()
 
-    # Check and replace IDs for Assets
-    def _match_asset_ids(self, verbal):
+    def _match_currencies(self):
         for asset in self._data[FOF.ASSETS]:
-            isin = asset['isin'] if 'isin' in asset else ''
-            reg_code = asset['reg_code'] if 'reg_code' in asset else ''
-            name = asset['name'] if 'name' in asset else ''
-            country_code = asset['country'] if 'country' in asset else ''
-            expiry = asset['expiry'] if 'expiry' in asset else 0
-            asset_id = JalDB().get_asset_id(asset['symbol'], isin=isin, reg_code=reg_code, name=name, expiry=expiry,
-                                            dialog_new=verbal)
+            if asset['type'] != FOF.ASSET_MONEY:
+                continue
+            symbol = self._find_in_list(self._data[FOF.SYMBOLS], "asset", asset['id'])
+            asset_id = JalDB().get_asset_id({'symbol': symbol['symbol'], 'type': self._asset_types[asset['type']]})
+            if asset_id is not None:
+                symbol['asset'] = -asset_id
+                old_id, asset['id'] = asset['id'], -asset_id
+                self._update_id("currency", old_id, asset_id)
+                self._update_id("asset", old_id, asset_id)     # TRANSFERS section may have currency in asset list
+
+    # Check and replace IDs for Assets matched by isin
+    def _match_asset_isin(self):
+        for asset in self._data[FOF.ASSETS]:
+            if asset['id'] < 0:  # already matched
+                continue
+            if 'isin' in asset:
+                asset_id = JalDB().get_asset_id({'isin': asset['isin']})
+                if asset_id is not None:
+                    old_id, asset['id'] = asset['id'], -asset_id
+                    self._update_id("asset", old_id, asset_id)
+
+    # Check and replace IDs for Assets matched by reg_number
+    def _match_asset_reg_number(self):
+        for asset in self._data[FOF.ASSETS_DATA]:
+            if asset['asset'] < 0:  # already matched
+                continue
+            if 'reg_number' in asset:
+                asset_id = JalDB().get_asset_id({'reg_number': asset['reg_number']})
+                if asset_id is not None:
+                    asset = self._find_in_list(self._data[FOF.ASSETS], "id", asset['asset'])
+                    old_id, asset['id'] = asset['id'], -asset_id
+                    self._update_id("asset", old_id, asset_id)
+
+    def _match_asset_symbol(self):
+        for symbol in self._data[FOF.SYMBOLS]:
+            if symbol['asset'] < 0:  # already matched
+                continue
+            asset = self._find_in_list(self._data[FOF.ASSETS], "id", symbol['asset'])
+            search_data = {'symbol': symbol['symbol'], 'type': self._asset_types[asset['type']]}
+            data = self._find_in_list(self._data[FOF.ASSETS_DATA], "asset", symbol['asset'])
+            if data is not None:
+                self._uppend_keys_from(search_data, data, ['expiry'])
+            asset_id = JalDB().get_asset_id(search_data)
             if asset_id is not None:
-                JalDB().update_asset_data(asset_id, asset['symbol'], isin, reg_code, country_code)
                 old_id, asset['id'] = asset['id'], -asset_id
                 self._update_id("asset", old_id, asset_id)
-                if asset['type'] == FOF.ASSET_MONEY:
-                    self._update_id("currency", old_id, asset_id)
 
     # Check and replace IDs for Accounts
     def _match_account_ids(self):
@@ -168,8 +212,8 @@ class Statement(QObject):   # derived from QObject to have proper string transla
 
     # Replace 'old_value' with 'new_value' in keys 'tag_name' of sections listed in mutable_sections
     def _update_id(self, tag_name, old_value, new_value):
-        mutable_sections = [FOF.ACCOUNTS, FOF.ASSETS, FOF.TRADES, FOF.TRANSFERS, FOF.CORP_ACTIONS, FOF.ASSET_PAYMENTS,
-                            FOF.INCOME_SPENDING]
+        mutable_sections = [FOF.ACCOUNTS, FOF.ASSETS, FOF.SYMBOLS, FOF.ASSETS_DATA, FOF.TRADES, FOF.TRANSFERS,
+                            FOF.CORP_ACTIONS, FOF.ASSET_PAYMENTS, FOF.INCOME_SPENDING]
         for section in mutable_sections:
             for element in self._data[section]:
                 for tag in element:
@@ -224,19 +268,13 @@ class Statement(QObject):   # derived from QObject to have proper string transla
 
     def _import_assets(self, assets):
         for asset in assets:
-            if asset['id'] < 0:
-                continue
             isin = asset['isin'] if 'isin' in asset else ''
-            reg_code = asset['reg_code'] if 'reg_code' in asset else ''
             name = asset['name'] if 'name' in asset else ''
             country_code = asset['country'] if 'country' in asset else ''
-            expiry = asset['expiry'] if 'expiry' in asset else 0
-            try:
-                source = self._sources[asset['exchange']]
-            except KeyError:
-                source = MarketDataFeed.NA
-            asset_id = JalDB().add_asset(asset['symbol'], name, self._asset_types[asset['type']], isin, expiry=expiry,
-                                         data_source=source, reg_code=reg_code, country_code=country_code)
+            if asset['id'] < 0:
+                JalDB().update_asset_data(-asset['id'], {'isin': isin, 'name': name, 'country': country_code})
+                continue
+            asset_id = JalDB().add_asset(self._asset_types[asset['type']], name, isin, country_code=country_code)
             if asset_id:
                 old_id, asset['id'] = asset['id'], -asset_id
                 self._update_id("asset", old_id, asset_id)
@@ -244,6 +282,29 @@ class Statement(QObject):   # derived from QObject to have proper string transla
                     self._update_id("currency", old_id, asset_id)
             else:
                 raise Statement_ImportError(self.tr("Can't create asset: ") + f"{asset}")
+
+    def _import_symbol_tickers(self, symbols):
+        for symbol in symbols:
+            if symbol['asset'] > 0:
+                raise Statement_ImportError(self.tr("Symbol ticker isn't linked to asset: ") + f"{symbol}")
+            if symbol['currency'] > 0:
+                raise Statement_ImportError(self.tr("Symbol currency isn't linked to asset: ") + f"{symbol}")
+            asset = self._find_in_list(self._data[FOF.ASSETS], "id", symbol['asset'])
+            note = symbol['note'] if 'note' in symbol else ''
+            if asset['type'] == FOF.ASSET_MONEY:
+                source = MarketDataFeed.CBR
+            else:
+                try:
+                    source = self._sources[symbol['note']]
+                except KeyError:
+                    source = MarketDataFeed.NA
+            JalDB().add_symbol(-symbol['asset'], symbol['symbol'], -symbol['currency'], note, data_source=source)
+
+    def _import_asset_data(self, data):
+        for detail in data:
+            if detail['asset'] > 0:
+                raise Statement_ImportError(self.tr("Asset data aren't linked to asset: ") + f"{detail}")
+            JalDB().update_asset_data(-detail['asset'], detail)
     
     def _import_accounts(self, accounts):
         for account in accounts:
@@ -337,19 +398,19 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             tax = payment['tax'] if 'tax' in payment else 0
             if payment['type'] == FOF.PAYMENT_DIVIDEND:
                 if payment['id'] > 0:  # New dividend
-                    JalDB().add_dividend(DividendSubtype.Dividend, payment['timestamp'], -payment['account'],
+                    JalDB().add_dividend(Dividend.Dividend, payment['timestamp'], -payment['account'],
                                          -payment['asset'], payment['amount'], payment['description'], tax=tax)
                 else:  # Dividend exists, only tax to be updated
                     JalDB().update_dividend_tax(-payment['id'], payment['tax'])
             elif payment['type'] == FOF.PAYMENT_INTEREST:
                 if 'number' not in payment:
                     payment['number'] = ''
-                JalDB().add_dividend(DividendSubtype.BondInterest, payment['timestamp'], -payment['account'],
+                JalDB().add_dividend(Dividend.BondInterest, payment['timestamp'], -payment['account'],
                                      -payment['asset'], payment['amount'], payment['description'], payment['number'],
                                      tax=tax)
             elif payment['type'] == FOF.PAYMENT_STOCK_DIVIDEND:
                 if payment['id'] > 0:  # New dividend
-                    JalDB().add_dividend(DividendSubtype.StockDividend, payment['timestamp'], -payment['account'],
+                    JalDB().add_dividend(Dividend.StockDividend, payment['timestamp'], -payment['account'],
                                          -payment['asset'], payment['amount'], payment['description'],
                                          payment['number'], tax=tax, price=payment['price'])
                 else:  # Dividend exists, only tax to be updated
@@ -392,3 +453,131 @@ class Statement(QObject):   # derived from QObject to have proper string transla
             if dialog.store_account:
                 self._previous_accounts[JalDB().get_account_currency(dialog.account_id)] = dialog.account_id
             return dialog.account_id
+
+    # Returns asset dictionary by asset id
+    def _asset(self, asset_id) -> dict:
+        asset = self._find_in_list(self._data[FOF.ASSETS], 'id', asset_id)
+        if asset is None:
+            raise Statement_ImportError(self.tr("Asset id not found") + f"{asset_id}")
+        return asset
+
+    # Helper function that takes list of dictionaries and returns one element where key=value
+    # exception is raised if multiple elements found
+    # Returns None if nothing was found in the list
+    def _find_in_list(self, data_list, key, value):
+        filtered = [x for x in data_list if key in x and x[key] == value]
+        if filtered:
+            if len(filtered) == 1:
+                return filtered[0]
+            else:
+                raise Statement_ImportError(self.tr("Multiple match for ") + f"'{key}'='{value}'")
+
+    # Method finds currency in current statement data. New currency is created if no currency was found.
+    # Returns currency id
+    def currency_id(self, currency_symbol) -> int:
+        match = [x for x in self._data[FOF.SYMBOLS] if
+                 x['symbol'] == currency_symbol and self._asset(x['asset'])['type'] == FOF.ASSET_MONEY]
+        if match:
+            if len(match) == 1:
+                return match[0]["asset"]
+            else:
+                raise Statement_ImportError(self.tr("Multiple currency match for ") + f"{currency_symbol}")
+        else:
+            asset_id = max([0] + [x['id'] for x in self._data[FOF.ASSETS]]) + 1
+            self._data[FOF.ASSETS].append({"id": asset_id, "type": "money", "name": ""})
+            symbol_id = max([0] + [x['id'] for x in self._data[FOF.SYMBOLS]]) + 1
+            currency = {"id": symbol_id, "asset": asset_id, "symbol": currency_symbol,
+                        "currency": -JalSettings().getValue('BaseCurrency')}
+            self._data[FOF.SYMBOLS].append(currency)
+            return asset_id
+
+    # Method finds asset in current statement data. New asset is created if no asset was found.
+    # Returns asset id
+    def asset_id(self, asset_info) -> int:
+        asset = None
+        asset_info = {k: v for k, v in asset_info.items() if v}  # drop keys with empty values
+        if 'isin' in asset_info:
+            asset = self._find_in_list(self._data[FOF.ASSETS], 'isin', asset_info['isin'])
+        if asset is None and 'reg_number' in asset_info:
+            asset_data = self._find_in_list(self._data[FOF.ASSETS_DATA], 'reg_number', asset_info['reg_number'])
+            if asset_data is not None:
+                asset = self._find_in_list(self._data[FOF.ASSETS], 'id', asset_data['asset'])
+        if asset is None and 'symbol' in asset_info:
+            symbol = self._find_in_list(self._data[FOF.SYMBOLS], 'symbol', asset_info['symbol'])
+            if symbol is not None:
+                asset = self._find_in_list(self._data[FOF.ASSETS], 'id', symbol['asset'])
+                if 'isin' in asset and 'isin' in asset_info and asset['isin'] != asset_info['isin']:
+                    asset = None
+        if asset is None and 'search_online' in asset_info:
+            if asset_info['search_online'] == "MOEX":
+                search_data = {}
+                self._uppend_keys_from(search_data, asset_info, ['isin', 'reg_number'])
+                symbol = QuoteDownloader.MOEX_find_secid(**search_data)
+                if not symbol and 'symbol' in asset_info:
+                    symbol = asset_info['symbol']
+                currency = asset_info['currency'] if 'currency' in asset_info else None  # Keep currency
+                asset_info = QuoteDownloader.MOEX_info(symbol=symbol)
+                asset_info['type'] = FOF.convert_predefined_asset_type(asset_info['type'])
+                if currency is not None:
+                    asset_info['currency'] = currency
+                asset_info['note'] = "MOEX"
+                return self.asset_id(asset_info)  # Call itself once again to cross-check downloaded data
+        if asset is None:
+            if 'should_exist' in asset_info and asset_info['should_exist']:
+                raise Statement_ImportError(self.tr("Can't locate asset in statement data: ") + f"'{asset_info}'")
+            asset_id = max([0] + [x['id'] for x in self._data[FOF.ASSETS]]) + 1
+            asset = {"id": asset_id}
+            self._uppend_keys_from(asset, asset_info, ['type', 'name', 'isin', 'country'])
+            self._data[FOF.ASSETS].append(asset)
+            if 'symbol' in asset_info:
+                symbol_id = max([0] + [x['id'] for x in self._data[FOF.SYMBOLS]]) + 1
+                symbol = {"id": symbol_id, "asset": asset_id}
+                self._uppend_keys_from(symbol, asset_info, ['symbol', 'currency', 'note'])
+                self._data[FOF.SYMBOLS].append(symbol)
+            data = {}
+            self._uppend_keys_from(data, asset_info, ['reg_number', 'expiry', 'principal'])
+            if data:
+                data_id = max([0] + [x['id'] for x in self._data[FOF.ASSETS_DATA]]) + 1
+                data['id'] = data_id
+                data['asset'] = asset_id
+                self._data[FOF.ASSETS_DATA].append(data)
+        else:
+            if 'type' in asset and asset['type'] != FOF.ASSET_MONEY:
+                self.update_asset_data(asset['id'], asset_info)
+        return asset['id']
+
+    # takes key from keys one by one and copies it from src to dst if it exists in src
+    def _uppend_keys_from(self, dst, src, keys):
+        for key in keys:
+            if key in src:
+                dst[key] = src[key]
+
+    def update_asset_data(self, asset_id, asset_info):
+        asset = self._find_in_list(self._data[FOF.ASSETS], "id", asset_id)
+        self._uppend_keys_from(asset, asset_info, ['name', 'isin', 'country'])
+        # Add new asset symbol if information provided
+        if 'symbol' in asset_info:
+            symbol_exists = False
+            symbols = [x for x in self._data[FOF.SYMBOLS] if "asset" in x and x["asset"] == asset_id]
+            if symbols:
+                for symbol in symbols:
+                    if symbol['symbol'] == asset_info['symbol'] and (
+                            'currency' not in asset_info or symbol['currency'] == asset_info['currency']):
+                        symbol_exists = True
+            if not symbol_exists:
+                symbol_id = max([0] + [x['id'] for x in self._data[FOF.SYMBOLS]]) + 1
+                symbol = {"id": symbol_id, "asset": asset_id}
+                self._uppend_keys_from(symbol, asset_info, ['symbol', 'currency', 'note', 'alt_symbol'])
+                self._data[FOF.SYMBOLS].append(symbol)
+        # Update the rest of asset data
+        asset_data = self._find_in_list(self._data[FOF.ASSETS_DATA], "asset", asset_id)
+        if asset_data is None:
+            if {'reg_number', 'expiry', 'principal'}.intersection(set(asset_info)):  # if keys are present in info
+                asset_data = {}
+                data_id = max([0] + [x['id'] for x in self._data[FOF.ASSETS_DATA]]) + 1
+                asset_data['id'] = data_id
+                asset_data['asset'] = asset_id
+                self._data[FOF.ASSETS_DATA].append(asset_data)
+            else:
+                return
+        self._uppend_keys_from(asset_data, asset_info, ['reg_number', 'expiry'])
