@@ -1,54 +1,11 @@
 import logging
 from datetime import datetime
-from PySide6.QtWidgets import QApplication, QDialog
-from PySide6.QtSql import QSqlTableModel
+from PySide6.QtWidgets import QApplication
 
-from jal.ui.ui_add_asset_dlg import Ui_AddAssetDialog
-from jal.constants import Setup, BookAccount, PredefindedAccountType, PredefinedAsset
+from jal.constants import Setup, BookAccount, PredefindedAccountType, AssetData, MarketDataFeed, PredefinedAsset
 from jal.db.helpers import db_connection, executeSQL, readSQL, get_country_by_code
 
 
-# -----------------------------------------------------------------------------------------------------------------------
-class AddAssetDialog(QDialog, Ui_AddAssetDialog):
-    def __init__(self, symbol, isin='', name=''):
-        QDialog.__init__(self)
-        self.setupUi(self)
-        self.asset_id = None
-
-        self.SymbolEdit.setText(symbol)
-        self.isinEdit.setText(isin)
-        self.NameEdit.setText(name)
-
-        self.type_model = QSqlTableModel(db=db_connection())
-        self.type_model.setTable('asset_types')
-        self.type_model.select()
-        self.TypeCombo.setModel(self.type_model)
-        self.TypeCombo.setModelColumn(1)
-
-        self.data_src_model = QSqlTableModel(db=db_connection())
-        self.data_src_model.setTable('data_sources')
-        self.data_src_model.select()
-        self.DataSrcCombo.setModel(self.data_src_model)
-        self.DataSrcCombo.setModelColumn(1)
-
-        # center dialog with respect to main application window
-        parent = None
-        for widget in QApplication.topLevelWidgets():
-            if widget.objectName() == Setup.MAIN_WND_NAME:
-                parent = widget
-        if parent:
-            x = parent.x() + parent.width() / 2 - self.width() / 2
-            y = parent.y() + parent.height() / 2 - self.height() / 2
-            self.setGeometry(x, y, self.width(), self.height())
-
-    def accept(self):
-        self.asset_id = JalDB().add_asset(self.SymbolEdit.text(), self.NameEdit.text(),
-                                          self.type_model.record(self.TypeCombo.currentIndex()).value("id"),
-                                          self.isinEdit.text(),
-                                          self.data_src_model.record(self.DataSrcCombo.currentIndex()).value("id"))
-        super().accept()
-
-
 # ----------------------------------------------------------------------------------------------------------------------
 class JalDB:
     def __init__(self):
@@ -66,12 +23,20 @@ class JalDB:
     def get_language_code(self, language_id):
         return readSQL("SELECT language FROM languages WHERE id = :language_id", [(':language_id', language_id)])
 
-    def get_asset_name(self, asset_id):
-        return readSQL("SELECT name FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
+    def get_asset_name(self, asset_id, full=False):
+        if full:
+            return readSQL("SELECT full_name FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
+        else:   # FIXME Below query may return several symbols
+            return readSQL("SELECT symbol FROM assets AS a LEFT JOIN asset_tickers AS s "   
+                           "ON s.asset_id=a.id AND s.active=1 WHERE a.id=:asset_id", [(":asset_id", asset_id)])
 
     def get_asset_type(self, asset_id):
         return readSQL("SELECT type_id FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
 
+    def get_asset_country(self, asset_id):
+        return readSQL("SELECT c.name FROM assets AS a LEFT JOIN countries AS c ON c.id=a.country_id "
+                       "WHERE a.id=:asset_id", [(":asset_id", asset_id)])
+
     def get_account_name(self, account_id):
         return readSQL("SELECT name FROM accounts WHERE id=:account_id", [(":account_id", account_id)])
 
@@ -80,14 +45,13 @@ class JalDB:
     def get_account_id(self, accountNumber, accountCurrency=''):
         if accountCurrency:
             account_id = readSQL("SELECT a.id FROM accounts AS a "
-                                 "LEFT JOIN assets AS c ON c.id=a.currency_id "
-                                 "WHERE a.number=:account_number AND c.name=:currency_name",
+                                 "LEFT JOIN currencies AS c ON c.id=a.currency_id "
+                                 "WHERE a.number=:account_number AND c.symbol=:currency_name",
                                  [(":account_number", accountNumber), (":currency_name", accountCurrency)],
                                  check_unique=True)
         else:
-            account_id = readSQL("SELECT a.id FROM accounts AS a "
-                                 "LEFT JOIN assets AS c ON c.id=a.currency_id "
-                                 "WHERE a.number=:account_number", [(":account_number", accountNumber)],
+            account_id = readSQL("SELECT a.id FROM accounts AS a WHERE a.number=:account_number",
+                                 [(":account_number", accountNumber)],
                                  check_unique=True)
         return account_id
 
@@ -95,29 +59,27 @@ class JalDB:
         return readSQL("SELECT id FROM accounts WHERE number=:account_number AND currency_id=:currency",
                        [(":account_number", account_number), (":currency", currency_code)], check_unique=True)
 
-    def add_account(self, account_number, currency_code, account_type=PredefindedAccountType.Investment):
-        account_id = self.find_account(account_number, currency_code)
+    def add_account(self, account_number, currency_id, account_type=PredefindedAccountType.Investment):
+        account_id = self.find_account(account_number, currency_id)
         if account_id:  # Account already exists
             logging.warning(self.tr("Account already exists: ") +
-                            f"{account_number} ({self.get_asset_name(currency_code)})")
+                            f"{account_number} ({self.get_asset_name(currency_id)})")
             return account_id
-        currency = self.get_asset_name(currency_code)
-        account_info = readSQL(
-            "SELECT a.name AS name, SUBSTR(a.name, 1, LENGTH(a.name)-LENGTH(c.name)-1) AS short_name, "
-            "SUBSTR(a.name, -(LENGTH(c.name)+1), LENGTH(c.name)+1) = '.'||c.name AS auto_name "
-            "FROM accounts AS a LEFT JOIN assets AS c ON a.currency_id = c.id WHERE number=:account_number LIMIT 1",
-            [(":account_number", account_number)], named=True)
-        if account_info:  # Account with the same number but different currency exists
-            if account_info['auto_name']:
-                new_name = account_info['short_name'] + '.' + currency
+        currency = self.get_asset_name(currency_id)
+        account = readSQL("SELECT a.name, a.organization_id, a.country_id, c.symbol AS currency FROM accounts a "
+                          "LEFT JOIN assets_ext c ON c.id=a.currency_id WHERE a.number=:number LIMIT 1",
+                          [(":number", account_number)], named=True)
+        if account:  # Account with the same number but different currency exists
+            if account['name'][-len(account['currency']):] == account['currency']:
+                new_name = account['name'][:-len(account['currency'])] + currency
             else:
-                new_name = account_info['name'] + '.' + currency
+                new_name = account['name'] + '.' + currency
             query = executeSQL(
                 "INSERT INTO accounts (type_id, name, active, number, currency_id, organization_id, country_id) "
                 "SELECT a.type_id, :new_name, a.active, a.number, :currency_id, a.organization_id, a.country_id "
                 "FROM accounts AS a LEFT JOIN assets AS c ON c.id=:currency_id "
                 "WHERE number=:account_number LIMIT 1",
-                [(":account_number", account_number), (":currency_id", currency_code), (":new_name", new_name)])
+                [(":account_number", account_number), (":currency_id", currency_id), (":new_name", new_name)])
             return query.lastInsertId()
 
         bank_name = self.tr("Bank for #" + account_number)
@@ -128,7 +90,7 @@ class JalDB:
         query = executeSQL("INSERT INTO accounts (type_id, name, active, number, currency_id, organization_id) "
                            "VALUES(:type, :name, 1, :number, :currency, :bank)",
                            [(":type", account_type), (":name", account_number+'.'+currency),
-                            (":number", account_number), (":currency", currency_code), (":bank", bank_id)])
+                            (":number", account_number), (":currency", currency_id), (":bank", bank_id)])
         return query.lastInsertId()
 
     def get_account_currency(self, account_id):
@@ -137,105 +99,134 @@ class JalDB:
     def get_account_bank(self, account_id):
         return readSQL("SELECT organization_id FROM accounts WHERE id=:account_id", [(":account_id", account_id)])
 
-    # Searches for asset_id in database - first by ISIN, then by Reg.Code next by Symbol
-    # If found - tries to update data if some is empty in database
-    # If asset not found and 'dialog_new' is True - pops up a window for asset creation
-    # Returns: asset_id or None if new asset creation failed
-    def get_asset_id(self, symbol, isin='', reg_code='', name='', expiry=0, dialog_new=True):  # TODO Change params to **kwargs
+    # Searches for asset_id in database based on keys available in search data:
+    # first by 'isin', then by 'reg_number', next by 'symbol' and other
+    # Returns: asset_id or None if not found
+    def get_asset_id(self, search_data):
         asset_id = None
-        if isin:
-            asset_id = readSQL("SELECT id FROM assets WHERE isin=:isin", [(":isin", isin)])
-            if asset_id is None:
-                asset_id = readSQL("SELECT id FROM assets WHERE name=:symbol COLLATE NOCASE AND coalesce(isin, '')=''",
-                                   [(":symbol", symbol)])
-        if asset_id is None:
-            if reg_code:
-                asset_id = readSQL("SELECT asset_id FROM asset_reg_id WHERE reg_code=:reg_code",
-                                   [(":reg_code", reg_code)])
-            if asset_id is None:
-                asset_id = readSQL("SELECT id FROM assets WHERE name=:symbol AND "
-                                   "((expiry=:expiry AND type_id=:derivative) OR type_id<>:derivative) COLLATE NOCASE",
-                                   [(":symbol", symbol), (":expiry", expiry),
-                                    (":derivative", PredefinedAsset.Derivative)])
-        if asset_id is None and dialog_new:
-            dialog = AddAssetDialog(symbol, isin=isin, name=name)
-            dialog.exec()
-            asset_id = dialog.asset_id
+        if 'isin' in search_data and search_data['isin']:
+            asset_id = readSQL("SELECT id FROM assets WHERE isin=:isin", [(":isin", search_data['isin'])])
+        if asset_id is None and 'reg_number' in search_data and search_data['reg_number']:
+            asset_id = readSQL("SELECT asset_id FROM asset_data WHERE datatype=:datatype AND value=:reg_number",
+                               [(":datatype", AssetData.RegistrationCode), (":reg_number", search_data['reg_number'])])
+        if asset_id is None and 'symbol' in search_data and search_data['symbol']:
+            if 'type' in search_data:
+                if 'expiry' in search_data:
+                    asset_id = readSQL("SELECT a.id FROM assets_ext a "
+                                       "LEFT JOIN asset_data d ON a.id=d.asset_id AND d.datatype=:datatype "
+                                       "WHERE symbol=:symbol COLLATE NOCASE AND type_id=:type AND value=:value",
+                                       [(":datatype", AssetData.ExpiryDate), (":symbol", search_data['symbol']),
+                                        (":type", search_data['type']), (":expiry", search_data['expiry'])])
+                else:
+                    asset_id = readSQL("SELECT id FROM assets_ext "
+                                       "WHERE symbol=:symbol COLLATE NOCASE and type_id=:type",
+                                       [(":symbol", search_data['symbol']), (":type", search_data['type'])])
+            else:
+                asset_id = readSQL("SELECT id FROM assets_ext WHERE symbol=:symbol COLLATE NOCASE",
+                                   [(":symbol", search_data['symbol'])])
         return asset_id
 
-    def update_asset_data(self, asset_id, new_symbol='', new_isin='', new_reg='', new_country_code='', expiry=0):  # TODO Change params to **kwargs
-        if new_symbol:
-            symbol = readSQL("SELECT name FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
-            if new_symbol.upper() != symbol.upper():
-                _ = executeSQL("UPDATE assets SET name=:symbol WHERE id=:asset_id",
-                               [(":symbol", new_symbol), (":asset_id", asset_id)])
-                # Show warning only if symbol was changed not due known bankruptcy or new issue pattern
-                if not (((symbol == new_symbol[:-1]) and (new_symbol[-1] == 'D' or new_symbol[-1] == 'Q')) or
-                        ((symbol[:-1] == new_symbol) and (symbol[-1] == 'D' or symbol[-1] == 'Q'))):
-                    logging.info(self.tr("Symbol updated ") + f"{symbol} -> {new_symbol}")
-        if new_isin:
-            isin = readSQL("SELECT isin FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
-            if isin == '':
+    def get_quote(self, asset_id, currency_id, timestamp):
+        return readSQL("SELECT quote FROM quotes WHERE asset_id=:asset_id "
+                       "AND currency_id=:currency_id AND timestamp=:timestamp",
+                       [(":asset_id", asset_id), (":currency_id", currency_id), (":timestamp", timestamp)])
+
+    # Set quotations for given asset_id and currency_id
+    # quotations is a list of {'timestamp', 'quote'} values
+    def update_quotes(self, asset_id, currency_id, quotations):
+        data = [x for x in quotations if x['timestamp'] is not None and x['quote'] is not None]  # Drop Nones
+        if data:
+            for quote in quotations:
+                _ = executeSQL("INSERT OR REPLACE INTO quotes (asset_id, currency_id, timestamp, quote) "
+                               "VALUES(:asset_id, :currency_id, :timestamp, :quote)",
+                               [(":asset_id", asset_id), (":currency_id", currency_id),
+                                (":timestamp", quote['timestamp']), (":quote", quote['quote'])])
+            begin = min(data, key=lambda x: x['timestamp'])['timestamp']
+            end = max(data, key=lambda x: x['timestamp'])['timestamp']
+            logging.info(self.tr("Quotations were updated: ") +
+                         f"{self.get_asset_name(asset_id)} ({self.get_asset_name(asset_id)}) " 
+                         f"{datetime.utcfromtimestamp(begin).strftime('%d/%m/%Y')} - "
+                         f"{datetime.utcfromtimestamp(end).strftime('%d/%m/%Y')}")
+
+    def add_asset(self, asset_type, name, isin, country_code=''):
+        country_id = get_country_by_code(country_code)
+        query = executeSQL("INSERT INTO assets (type_id, full_name, isin, country_id) "
+                           "VALUES (:type, :full_name, :isin, :country_id)",
+                           [(":type", asset_type), (":full_name", name),
+                            (":isin", isin), (":country_id", country_id)], commit=True)
+        return query.lastInsertId()
+
+    def add_symbol(self, asset_id, symbol, currency, note, data_source=MarketDataFeed.NA):
+        existing = readSQL("SELECT id, symbol, description, quote_source FROM asset_tickers "
+                           "WHERE asset_id=:asset_id AND symbol=:symbol AND currency_id=:currency",
+                           [(":asset_id", asset_id), (":symbol", symbol), (":currency", currency)], named=True)
+        if existing is None:   # Create new symbol
+            _ = executeSQL("INSERT INTO asset_tickers (asset_id, symbol, currency_id, description, quote_source) "
+                           "VALUES (:asset_id, :symbol, :currency, :note, :data_source)",
+                           [(":asset_id", asset_id), (":symbol", symbol), (":currency", currency),
+                            (":note", note), (":data_source", data_source)])
+        else:   # Update data for existing symbol
+            if not existing['description']:
+                _ = executeSQL("UPDATE asset_tickers SET description=:note WHERE id=:id",
+                               [(":note", note), (":id", existing['id'])])
+            if existing['quote_source'] == MarketDataFeed.NA:
+                _ = executeSQL("UPDATE asset_tickers SET quote_source=:data_source WHERE id=:id",
+                               [(":data_source", data_source), (":id", existing['id'])])
+
+    def update_asset_data(self, asset_id, data):
+        asset = readSQL("SELECT type_id, isin, full_name FROM assets WHERE id=:asset_id",
+                        [(":asset_id", asset_id)], named=True)
+        if asset is None:
+            logging.warning(self.tr("Asset not found for update: ") + f"{data}")
+            return
+        if 'isin' in data and data['isin']:
+            if asset['isin']:
+                if asset['isin'] != data['isin']:
+                    logging.error(self.tr("Unexpected attempt to update ISIN for ")
+                                  + f"{self.get_asset_name(asset_id)}: {asset['isin']} -> {data['isin']}")
+            else:
                 _ = executeSQL("UPDATE assets SET isin=:new_isin WHERE id=:asset_id",
-                               [(":new_isin", new_isin), (":asset_id", asset_id)])
-            elif isin != new_isin:
-                logging.warning(self.tr("ISIN mismatch for ")
-                                + f"{self.get_asset_name(asset_id)}: {isin} != {new_isin}")
-        if new_reg:
-            reg = readSQL("SELECT reg_code FROM asset_reg_id WHERE asset_id=:asset_id", [(":asset_id", asset_id)])
-            if new_reg != reg:
-                _ = executeSQL("INSERT OR REPLACE INTO asset_reg_id(asset_id, reg_code) VALUES(:asset_id, :new_reg)",
-                               [(":new_reg", new_reg), (":asset_id", asset_id)])
-                logging.info(self.tr("Reg.number updated for ")
-                             + f"{self.get_asset_name(asset_id)}: {reg} -> {new_reg}")
-        if new_country_code:
+                               [(":new_isin", data['isin']), (":asset_id", asset_id)])
+        if 'name' in data and data['name']:
+            if not asset['full_name']:
+                _ = executeSQL("UPDATE assets SET full_name=:new_name WHERE id=:asset_id",
+                               [(":new_name", data['name']), (":asset_id", asset_id)])
+        if 'country' in data and data['country']:
             country_id, country_code = readSQL("SELECT a.country_id, c.code FROM assets AS a LEFT JOIN countries AS c "
-                                   "ON a.country_id=c.id WHERE a.id=:asset_id", [(":asset_id", asset_id)])
-            if (country_id == 0) or (country_code.lower() != new_country_code.lower()):
-                new_country_id = get_country_by_code(new_country_code)
+                                               "ON a.country_id=c.id WHERE a.id=:asset_id", [(":asset_id", asset_id)])
+            if (country_id == 0) or (country_code.lower() != data['country'].lower()):
+                new_country_id = get_country_by_code(data['country'])
                 _ = executeSQL("UPDATE assets SET country_id=:new_country_id WHERE id=:asset_id",
                                [(":new_country_id", new_country_id), (":asset_id", asset_id)])
                 if country_id != 0:
                     logging.info(self.tr("Country updated for ")
-                                 + f"{self.get_asset_name(asset_id)}: {country_code} -> {new_country_code}")
-        if expiry:
-            _ = executeSQL("UPDATE assets SET expiry=:expiry WHERE id=:asset_id",
-                           [(":expiry", expiry), (":asset_id", asset_id)])
-
-    def get_quote(self, asset_id, timestamp):
-        return readSQL("SELECT quote FROM quotes WHERE asset_id=:asset_id AND timestamp=:timestamp",
-                       [(":asset_id", asset_id), (":timestamp", timestamp)])
-
-    def update_quote(self, asset_id, timestamp, quote):
-        if (timestamp is None) or (quote is None):
-            return
-        old_id = 0
-        query = executeSQL("SELECT id FROM quotes WHERE asset_id = :asset_id AND timestamp = :timestamp",
-                           [(":asset_id", asset_id), (":timestamp", timestamp)])
-        if query.next():
-            old_id = query.value(0)
-        if old_id:
-            executeSQL("UPDATE quotes SET quote=:quote WHERE id=:old_id", [(":quote", quote), (":old_id", old_id), ])
-        else:
-            executeSQL("INSERT INTO quotes(timestamp, asset_id, quote) VALUES (:timestamp, :asset_id, :quote)",
-                       [(":timestamp", timestamp), (":asset_id", asset_id), (":quote", quote)])
-        logging.info(self.tr("Quote loaded: ") + f"{self.get_asset_name(asset_id)} " 
-                     f"@ {datetime.utcfromtimestamp(timestamp).strftime('%d/%m/%Y %H:%M:%S')} = {quote}")
-
-    def add_asset(self, symbol, name, asset_type, isin, data_source=-1, reg_code=None, country_code='', expiry=0):  # TODO Change params to **kwargs
-        country_id = get_country_by_code(country_code)
-        query = executeSQL("INSERT INTO assets(name, type_id, full_name, isin, src_id, country_id, expiry) "
-                           "VALUES(:symbol, :type, :full_name, :isin, :data_src, :country_id, :expiry)",
-                           [(":symbol", symbol), (":type", asset_type), (":full_name", name),
-                            (":isin", isin), (":data_src", data_source), (":country_id", country_id),
-                            (":expiry", expiry)], commit=True)
-        asset_id = query.lastInsertId()
-        if asset_id is None:
-            logging.error(self.tr("Failed to add new asset: ") + f"{symbol}")
-        if reg_code is not None:
-            _ = executeSQL("INSERT INTO asset_reg_id(asset_id, reg_code) VALUES(:asset_id, :new_reg)",
-                           [(":new_reg", reg_code), (":asset_id", asset_id)])
-        return asset_id
+                                 + f"{self.get_asset_name(asset_id)}: {country_code} -> {data['country']}")
+        if 'reg_number' in data and data['reg_number']:
+            reg_number = readSQL("SELECT value FROM asset_data WHERE datatype=:datatype AND asset_id=:asset_id",
+                                 [(":datatype", AssetData.RegistrationCode), (":asset_id", asset_id)])
+            if reg_number != data['reg_number']:
+                _ = executeSQL("INSERT OR REPLACE INTO asset_data(asset_id, datatype, value) "
+                               "VALUES(:asset_id, :datatype, :reg_number)",
+                               [(":asset_id", asset_id), (":datatype", AssetData.RegistrationCode),
+                                (":reg_number", data['reg_number'])])
+                if reg_number:
+                    logging.info(self.tr("Reg.number updated for ")
+                                 + f"{self.get_asset_name(asset_id)}: {reg_number} -> {data['reg_number']}")
+        if 'expiry' in data and data['expiry']:
+            _ = executeSQL("INSERT OR REPLACE INTO asset_data(asset_id, datatype, value) "
+                           "VALUES(:asset_id, :datatype, :expiry)",
+                           [(":asset_id", asset_id), (":datatype", AssetData.ExpiryDate),
+                            (":expiry", data['expiry'])])
+        if 'principal' in data and asset['type_id'] == PredefinedAsset.Bond:
+            try:
+                principal = float(data['principal'])
+                if principal > 0:
+                    _ = executeSQL("INSERT OR REPLACE INTO asset_data(asset_id, datatype, value) "
+                                   "VALUES(:asset_id, :datatype, :principal)",
+                                   [(":asset_id", asset_id), (":datatype", AssetData.PrincipalValue),
+                                    (":principal", principal)])
+            except ValueError:
+                pass
 
     def add_dividend(self, subtype, timestamp, account_id, asset_id, amount, note, trade_number='',
                      tax=0.0, price=None):
@@ -253,7 +244,8 @@ class JalDB:
                         (":tax", tax), (":note", note)],
                        commit=True)
         if price is not None:
-            self.update_quote(asset_id, timestamp, price)
+            self.update_quotes(asset_id, self.get_account_currency(account_id),
+                               [{'timestamp': timestamp, 'quote': price}])
 
     def update_dividend_tax(self, dividend_id, new_tax):
         _ = executeSQL("UPDATE dividends SET tax=:tax WHERE id=:dividend_id",
@@ -339,6 +331,13 @@ class JalDB:
         _ = executeSQL("UPDATE accounts SET reconciled_on=:timestamp WHERE id = :account_id",
                        [(":timestamp", timestamp), (":account_id", account_id)])
 
+    def account_reconciliation_timestamp(self, account_id):
+        timestamp = readSQL("SELECT reconciled_on FROM accounts WHERE id=:account_id", [(":account_id", account_id)])
+        if timestamp is None:
+            return 0
+        else:
+            return timestamp
+
     def get_asset_amount(self, timestamp, account_id, asset_id):
         return readSQL("SELECT amount_acc FROM ledger "
                        "WHERE account_id=:account_id AND asset_id=:asset_id AND timestamp<=:timestamp "
