@@ -104,7 +104,7 @@ def executeSQL(sql_text, params=[], forward_only=True, commit=False):
 def readSQL(sql_text, params=None, named=False, check_unique=False):
     if params is None:
         params = []
-    query = QSqlQuery(db_connection())
+    query = QSqlQuery(db_connection())   # TODO reimplement via ExecuteSQL() call in order to get rid of duplicated code
     query.setForwardOnly(True)
     if not query.prepare(sql_text):
         logging.error(f"SQL prep: '{query.lastError().text()}' for query '{sql_text}' | '{params}'")
@@ -235,27 +235,9 @@ def get_country_by_code(country_code):
         logging.warning(QApplication.translate('DB', "Unknown country code: ") + f"'{country_code}'")
     return country_id
 
-# -------------------------------------------------------------------------------------------------------------------
-# Function sets asset country if asset.country_id is 0
-# Shows warning and sets new asset country if asset.country_id was different before
-# Does nothing if asset country had already the same value
-def update_asset_country(asset_id, country_id):
-    old_id = readSQL("SELECT country_id FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
-    if old_id == country_id:
-        return
-    _ = executeSQL("UPDATE assets SET country_id=:country_id WHERE id=:asset_id",
-                   [(":asset_id", asset_id), (":country_id", country_id)])
-    if old_id == 0:
-        return
-    old_country = readSQL("SELECT name FROM countries WHERE id=:country_id", [(":country_id", old_id)])
-    new_country = readSQL("SELECT name FROM countries WHERE id=:country_id", [(":country_id", country_id)])
-    asset_name = readSQL("SELECT name FROM assets WHERE id=:asset_id", [(":country_id", asset_id)])
-    logging.warning(QApplication.translate('DB', "Country was changed for asset ") +
-                    f"{asset_name}: f{old_country} -> {new_country}")
-
 # -------------------------------------------------------------------------------------------------------------------
 def account_last_date(account_id):
-    last_timestamp = readSQL("SELECT MAX(o.timestamp) FROM all_operations AS o "
+    last_timestamp = readSQL("SELECT MAX(o.timestamp) FROM operation_sequence AS o "
                              "LEFT JOIN accounts AS a ON o.account_id=a.id WHERE a.id=:account_id",
                              [(":account_id", account_id)])
     last_timestamp = 0 if last_timestamp == '' else last_timestamp
