@@ -54,7 +54,9 @@ class QuoteDownloader(QObject):
             MarketDataFeed.RU: self.MOEX_DataReader,
             MarketDataFeed.EU: self.Euronext_DataReader,
             MarketDataFeed.US: self.Yahoo_Downloader,
-            MarketDataFeed.CA: self.TMX_Downloader
+            MarketDataFeed.CA: self.TMX_Downloader,
+            MarketDataFeed.GB: self.YahooLSE_Downloader,
+            MarketDataFeed.FRA: self.YahooFRA_Downloader
         }
 
     def showQuoteDownloadDialog(self, parent):
@@ -63,31 +65,34 @@ class QuoteDownloader(QObject):
             self.UpdateQuotes(dialog.getStartDate(), dialog.getEndDate())
             self.download_completed.emit()
 
+    # FIXME This method may work incorrectly for assets with multiple symbols
     def UpdateQuotes(self, start_timestamp, end_timestamp):
         self.PrepareRussianCBReader()
         jal_db = JalDB()
         query = executeSQL("WITH _holdings AS ( "
-                           "SELECT l.asset_id AS asset FROM ledger AS l "
-                           "WHERE l.book_account = 4 AND l.timestamp <= :end_timestamp "
+                           "SELECT l.asset_id AS asset, l.account_id FROM ledger AS l "
+                           "WHERE l.book_account = :assets_book AND l.timestamp <= :end_timestamp "
                            "GROUP BY l.asset_id "
                            "HAVING SUM(l.amount) > :tolerance "
                            "UNION "
-                           "SELECT DISTINCT l.asset_id AS asset FROM ledger AS l "
+                           "SELECT DISTINCT l.asset_id AS asset, l.account_id FROM ledger AS l "
                            "WHERE l.book_account = :assets_book AND l.timestamp >= :start_timestamp "
                            "AND l.timestamp <= :end_timestamp "
                            "UNION "
-                           "SELECT DISTINCT a.currency_id AS asset FROM ledger AS l "
+                           "SELECT DISTINCT a.currency_id AS asset, NULL FROM ledger AS l "
                            "LEFT JOIN accounts AS a ON a.id = l.account_id "
                            "WHERE (l.book_account = :money_book OR l.book_account = :liabilities_book) "
                            "AND l.timestamp >= :start_timestamp AND l.timestamp <= :end_timestamp "
                            ") "
-                           "SELECT h.asset AS asset_id, a.name AS name, a.src_id AS feed_id, a.isin AS isin, "
+                           "SELECT h.asset AS asset_id, coalesce(ac.currency_id, 1) AS currency_id, "
+                           "a.symbol AS name, a.quote_source AS feed_id, a.isin AS isin, "
                            "MIN(q.timestamp) AS first_timestamp, MAX(q.timestamp) AS last_timestamp "
                            "FROM _holdings AS h "
-                           "LEFT JOIN assets AS a ON a.id=h.asset "
+                           "LEFT JOIN accounts AS ac ON ac.id=h.account_id "
+                           "LEFT JOIN assets_ext AS a ON a.id=h.asset AND a.currency_id=coalesce(ac.currency_id, 1) "
                            "LEFT JOIN quotes AS q ON q.asset_id=h.asset "
-                           "GROUP BY h.asset "
-                           "ORDER BY a.src_id",
+                           "GROUP BY h.asset, ac.currency_id "
+                           "ORDER BY feed_id",
                            [(":start_timestamp", start_timestamp), (":end_timestamp", end_timestamp),
                             (":assets_book", BookAccount.Assets), (":money_book", BookAccount.Money),
                             (":liabilities_book", BookAccount.Liabilities), (":tolerance", Setup.CALC_TOLERANCE)])
@@ -102,14 +107,16 @@ class QuoteDownloader(QObject):
             if end_timestamp < from_timestamp:
                 continue
             try:
-                data = self.data_loaders[asset['feed_id']](asset['asset_id'], asset['name'], asset['isin'],
-                                                           from_timestamp, end_timestamp)
+                data = self.data_loaders[asset['feed_id']](asset['asset_id'], asset['name'], asset['currency_id'],
+                                                           asset['isin'], from_timestamp, end_timestamp)
             except (xml_tree.ParseError, pd.errors.EmptyDataError, KeyError):
                 logging.warning(self.tr("No data were downloaded for ") + f"{asset}")
                 continue
             if data is not None:
+                quotations = []
                 for date, quote in data.iterrows():  # Date in pandas dataset is in UTC by default
-                    jal_db.update_quote(asset['asset_id'], int(date.timestamp()), float(quote[0]))
+                    quotations.append({'timestamp': int(date.timestamp()), 'quote': float(quote[0])})
+                jal_db.update_quotes(asset['asset_id'], asset['currency_id'], quotations)
         jal_db.commit()
         logging.info(self.tr("Download completed"))
 
@@ -126,14 +133,18 @@ class QuoteDownloader(QObject):
         self.CBR_codes = pd.DataFrame(rows, columns=["ISO_name", "CBR_code"])
 
     # Empty method to make a unified call for any asset
-    def Dummy_DataReader(self, _asset_id, _symbol, _isin, _start_timestamp, _end_timestamp):
+    def Dummy_DataReader(self, _asset_id, _symbol, _currency, _isin, _start_timestamp, _end_timestamp):
         return None
 
-    def CBR_DataReader(self, _asset_id, currency_code, _isin, start_timestamp, end_timestamp):
+    def CBR_DataReader(self, _asset_id, currency_code, _currency, _isin, start_timestamp, end_timestamp):
         date1 = datetime.utcfromtimestamp(start_timestamp).strftime('%d/%m/%Y')
         # add 1 day to end_timestamp as CBR sets rate are a day ahead
         date2 = (datetime.utcfromtimestamp(end_timestamp) + timedelta(days=1)).strftime('%d/%m/%Y')
-        code = str(self.CBR_codes.loc[self.CBR_codes["ISO_name"] == currency_code, "CBR_code"].values[0]).strip()
+        try:
+            code = str(self.CBR_codes.loc[self.CBR_codes["ISO_name"] == currency_code, "CBR_code"].values[0]).strip()
+        except IndexError:
+            logging.error(self.tr("Failed to get CBR data for: " + f"{currency_code}"))
+            return None
         url = f"http://www.cbr.ru/scripts/XML_dynamic.asp?date_req1={date1}&date_req2={date2}&VAL_NM_RQ={code}"
         xml_root = xml_tree.fromstring(get_web_data(url))
         rows = []
@@ -150,7 +161,7 @@ class QuoteDownloader(QObject):
 
     # Get asset data from http://www.moex.com
     # Accepts parameters:
-    #     symbol, isin, regcode - to identify asset
+    #     symbol, isin, reg_number - to identify asset
     #     special - if 'engine', 'market' and 'board' should be returned as part of result
     # Returns asset data or empty dictionary if nothing found
     @staticmethod
@@ -158,14 +169,15 @@ class QuoteDownloader(QObject):
         data = {}
         if 'symbol' in kwargs and not isEnglish(kwargs['symbol']):
             del kwargs['symbol']
+        currency = kwargs['currency'] if 'currency' in kwargs else ''
         # First try to load with symbol or isin from asset details API
         if 'symbol' in kwargs:
-            data = QuoteDownloader.MOEX_download_info(kwargs['symbol'])
+            data = QuoteDownloader.MOEX_download_info(kwargs['symbol'], currency=currency)
         if not data and 'isin' in kwargs:
-            data = QuoteDownloader.MOEX_download_info(kwargs['isin'])
+            data = QuoteDownloader.MOEX_download_info(kwargs['isin'], currency=currency)
         # If not found try to use search API with regnumber or isin
-        if not data and 'regnumber' in kwargs:
-            data = QuoteDownloader.MOEX_download_info(QuoteDownloader.MOEX_find_secid(regcode=kwargs['regnumber']))
+        if not data and 'reg_number' in kwargs:
+            data = QuoteDownloader.MOEX_download_info(QuoteDownloader.MOEX_find_secid(reg_number=kwargs['reg_number']))
         if not data and 'isin' in kwargs:
             data = QuoteDownloader.MOEX_download_info(QuoteDownloader.MOEX_find_secid(isin=kwargs['isin']))
         if 'special' not in kwargs:
@@ -179,13 +191,13 @@ class QuoteDownloader(QObject):
     # Searches for asset info on http://www.moex.com by symbol or ISIN provided as asset_code parameter
     # Returns disctionary with asset data: {symbol, isin, regnumber, engine, market, board, pricipal, expiry, etc}
     @staticmethod
-    def MOEX_download_info(asset_code) -> dict:
+    def MOEX_download_info(asset_code, currency='') -> dict:
         mapping = {
             'SECID': 'symbol',
             'NAME': 'name',
             'SHORTNAME': 'short_name',
             'ISIN': 'isin',
-            'REGNUMBER': 'reg_code',
+            'REGNUMBER': 'reg_number',
             'FACEVALUE': 'principal',
             'MATDATE': 'expiry',
             'LSTDELDATE': 'expiry',
@@ -228,49 +240,56 @@ class QuoteDownloader(QObject):
             logging.error(QApplication.translate("MOEX", "Unsupported MOEX security type: ") + f"{asset['type']}")
             return {}
 
-        for board in boards:
-            if board.attrib['is_primary'] == '1':
-                asset.update({'engine': board.attrib['engine'],
-                              'market': board.attrib['market'],
-                              'board': board.attrib['boardid']})
-                break
+        boards_list = [board.attrib for board in boards]
+        primary_board = [x for x in boards_list if "is_primary" in x and x["is_primary"] == '1']
+        if primary_board:
+            if currency == 'USD':
+                board_id = primary_board[0]['boardid'][:-1] + 'D'
+            elif currency == 'EUR':
+                board_id = primary_board[0]['boardid'][:-1] + 'E'
+            else:
+                board_id = primary_board[0]['boardid']
+            board = [x for x in boards_list if "boardid" in x and x["boardid"] == board_id]
+            if board:
+                asset.update({'engine': board[0]['engine'],
+                              'market': board[0]['market'],
+                              'board': board[0]['boardid']})
         return asset
 
-    # Searches for asset info on http://www.moex.com by given reg.number or isin
+    # Searches for asset info on http://www.moex.com by given reg_number or isin
     # Returns 'secid' if asset was found and empty string otherwise
     @staticmethod
     def MOEX_find_secid(**kwargs) -> str:
         secid = ''
-        try:
-            search_key = kwargs['regcode'] if 'regcode' in kwargs else kwargs['isin']
-        except KeyError:
-            return secid
-        if not search_key:
-            return secid
-        url = f"https://iss.moex.com/iss/securities.json?q={search_key}&iss.meta=off&limit=10"
-        asset_data = json.loads(get_web_data(url))
-        securities = asset_data['securities']
-        columns = securities['columns']
-        if 'regcode' in kwargs:
-            # Take only records that have given regnumber to get rid of additional issues
+        data = []
+        if 'reg_number' in kwargs:
+            url = f"https://iss.moex.com/iss/securities.json?q={kwargs['reg_number']}&iss.meta=off&limit=10"
+            asset_data = json.loads(get_web_data(url))
+            securities = asset_data['securities']
+            columns = securities['columns']
             data = [x for x in securities['data'] if
-                    x[columns.index('regnumber')] == search_key or x[columns.index('regnumber')] is None]
-        else:
+                    x[columns.index('regnumber')] == kwargs['reg_number'] or x[columns.index('regnumber')] is None]
+        if not data and 'isin' in kwargs:
+            url = f"https://iss.moex.com/iss/securities.json?q={kwargs['isin']}&iss.meta=off&limit=10"
+            asset_data = json.loads(get_web_data(url))
+            securities = asset_data['securities']
+            columns = securities['columns']
             data = securities['data']  # take the whole list if we search by isin
         if data:
             if len(data) > 1:
                 # remove not traded assets if there are any outdated doubles present
                 data = [x for x in data if x[columns.index('is_traded')] == 1]
             if len(data) > 1:  # and then check again
-                logging.info(QApplication.translate("MOEX", "Multiple MOEX assets found for reg.number: ") + search_key)
+                logging.info(QApplication.translate("MOEX", "Multiple MOEX assets found for: ") + f"{kwargs}")
                 return secid
             asset_data = dict(zip(columns, data[0]))
             secid = asset_data['secid'] if 'secid' in asset_data else ''
         return secid
 
     # noinspection PyMethodMayBeStatic
-    def MOEX_DataReader(self, asset_id, asset_code, isin, start_timestamp, end_timestamp, update_symbol=True):
-        asset = self.MOEX_info(symbol=asset_code, isin=isin, special=True)
+    def MOEX_DataReader(self, asset_id, asset_code, currency, isin, start_timestamp, end_timestamp, update_symbol=True):
+        currency = JalDB().get_asset_name(currency)
+        asset = self.MOEX_info(symbol=asset_code, isin=isin, currency=currency, special=True)
         if (asset['engine'] is None) or (asset['market'] is None) or (asset['board'] is None):
             logging.warning(f"Failed to find {asset_code} on moex.com")
             return None
@@ -278,12 +297,13 @@ class QuoteDownloader(QObject):
             asset_code = isin   # Corporate bonds are quoted by ISIN
         if (asset['market'] == 'shares') and (asset['board'] == 'TQIF'):
             asset_code = isin   # ETFs are quoted by ISIN
-
-        isin = asset['isin'] if 'isin' in asset else ''
-        reg_code = asset['reg_code'] if 'reg_code' in asset else ''
-        expiry = asset['expiry'] if 'expiry' in asset else 0
         if update_symbol:
-            JalDB().update_asset_data(asset_id, new_isin=isin, new_reg=reg_code, expiry=expiry)
+            isin = asset['isin'] if 'isin' in asset else ''
+            reg_number = asset['reg_number'] if 'reg_number' in asset else ''
+            expiry = asset['expiry'] if 'expiry' in asset else 0
+            principal = asset['principal'] if 'principal' in asset else 0
+            details = {'isin': isin, 'reg_number': reg_number, 'expiry': expiry, 'principal': principal}
+            JalDB().update_asset_data(asset_id, details)
 
         # Get price history
         date1 = datetime.utcfromtimestamp(start_timestamp).strftime('%Y-%m-%d')
@@ -307,7 +327,7 @@ class QuoteDownloader(QObject):
         return close
 
     # noinspection PyMethodMayBeStatic
-    def Yahoo_Downloader(self, _asset_id, asset_code, _isin, start_timestamp, end_timestamp):
+    def Yahoo_Downloader(self, _asset_id, asset_code, _currency, _isin, start_timestamp, end_timestamp):
         url = f"https://query1.finance.yahoo.com/v7/finance/download/{asset_code}?" \
               f"period1={start_timestamp}&period2={end_timestamp}&interval=1d&events=history"
         file = StringIO(get_web_data(url))
@@ -320,8 +340,16 @@ class QuoteDownloader(QObject):
         close = data.set_index("Date")
         return close
 
+    # The same as Yahoo_Downloader but it adds ".L" suffix to asset_code and returns prices in GBP
+    def YahooLSE_Downloader(self, asset_id, asset_code, currency, _isin, start_timestamp, end_timestamp):
+        return self.Yahoo_Downloader(asset_id, asset_code + '.L', currency, _isin, start_timestamp, end_timestamp)
+
+    # The same as Yahoo_Downloader but it adds ".F" suffix to asset_code and returns prices in EUR
+    def YahooFRA_Downloader(self, asset_id, asset_code, currency, _isin, start_timestamp, end_timestamp):
+        return self.Yahoo_Downloader(asset_id, asset_code + '.F', currency, _isin, start_timestamp, end_timestamp)
+
     # noinspection PyMethodMayBeStatic
-    def Euronext_DataReader(self, _asset_id, _asset_code, isin, start_timestamp, end_timestamp):
+    def Euronext_DataReader(self, _asset_id, _asset_code, _currency, isin, start_timestamp, end_timestamp):
         params = {'format': 'csv', 'decimal_separator': '.', 'date_form': 'd/m/Y', 'op': '', 'adjusted': '',
                   'base100': '', 'startdate': datetime.utcfromtimestamp(start_timestamp).strftime('%Y-%m-%d'),
                   'enddate': datetime.utcfromtimestamp(end_timestamp).strftime('%Y-%m-%d')}
@@ -350,7 +378,7 @@ class QuoteDownloader(QObject):
         return close
 
     # noinspection PyMethodMayBeStatic
-    def TMX_Downloader(self, _asset_id, asset_code, _isin, start_timestamp, end_timestamp):
+    def TMX_Downloader(self, _asset_id, asset_code, _currency, _isin, start_timestamp, end_timestamp):
         url = 'https://app-money.tmx.com/graphql'
         params = {
             "operationName": "getCompanyPriceHistoryForDownload",
@@ -380,3 +408,20 @@ class QuoteDownloader(QObject):
         close = data.set_index("Date")
         close.sort_index(inplace=True)
         return close
+
+    def updataData(self):
+        query = executeSQL("SELECT * FROM assets_ext WHERE quote_source!=:NA",
+                           [(":NA", MarketDataFeed.NA)])
+        while query.next():
+            asset = readSQLrecord(query, named=True)
+            if asset['type_id'] in [PredefinedAsset.Money, PredefinedAsset.Commodity, PredefinedAsset.Forex]:
+                continue
+            if asset['quote_source'] == MarketDataFeed.RU:
+                logging.info(self.tr("Checking MOEX data for: ") + asset['symbol'])
+                data = self.MOEX_info(symbol=asset['symbol'], isin=asset['isin'])
+                if data:
+                    if asset['full_name'] != data['name']:
+                        logging.info(self.tr("New full name found for:  ")
+                                     + f"{JalDB().get_asset_name(asset['id'])}: {asset['full_name']} -> {data['name']}")
+                    isin = data['isin'] if not asset['isin'] and 'isin' in data and data['isin'] else ''
+                    JalDB().update_asset_data(asset['id'], {'isin': isin})
