@@ -2,13 +2,14 @@ from PySide6.QtCore import Signal, Slot, Property
 from PySide6.QtWidgets import QApplication, QDialog, QWidget, QPushButton, QComboBox, QMenu, QHBoxLayout, QCheckBox, \
     QMessageBox
 from PySide6.QtSql import QSqlQuery, QSqlTableModel
-from jal.constants import Setup, PredefinedAsset
+from jal.constants import Setup
 from jal.db.db import JalDB
 from jal.db.settings import JalSettings
 from jal.db.helpers import db_connection, readSQL
 from jal.widgets.reference_dialogs import AccountListDialog
 from jal.ui.ui_select_account_dlg import Ui_SelectAccountDlg
 
+
 ########################################################################################################################
 #  UI Button to choose accounts
 ########################################################################################################################
@@ -107,13 +108,13 @@ class CurrencyComboBox(QComboBox):
         self.activated.connect(self.OnUserSelection)
 
         self.query = QSqlQuery(db=db_connection())
-        self.query.prepare(f"SELECT id, name FROM assets WHERE type_id={PredefinedAsset.Money}")
+        self.query.prepare(f"SELECT id, symbol FROM currencies")
         self.query.exec()
         self.model = QSqlTableModel(db=db_connection())
         self.model.setQuery(self.query)
         self.model.select()
         self.setModel(self.model)
-        self.setModelColumn(self.model.fieldIndex("name"))
+        self.setModelColumn(self.model.fieldIndex("symbol"))
 
     def isCustom(self):
         return True
@@ -125,7 +126,7 @@ class CurrencyComboBox(QComboBox):
         if self.p_selected_id == new_id:
             return
         self.p_selected_id = new_id
-        name = readSQL("SELECT name FROM currencies WHERE id = :id", [(":id", self.p_selected_id)])
+        name = readSQL("SELECT symbol FROM currencies WHERE id=:id", [(":id", self.p_selected_id)])
         if self.currentIndex() == self.findText(name):
             return
         self.setCurrentIndex(self.findText(name))
