@@ -6,8 +6,8 @@ from PySide6.QtWidgets import QLabel, QDateTimeEdit, QDateEdit, QLineEdit, QComb
 from jal.widgets.abstract_operation_details import AbstractOperationDetails
 from jal.widgets.reference_selector import AccountSelector, AssetSelector
 from jal.widgets.delegates import WidgetMapperDelegateBase
-from jal.constants import TransactionType, DividendSubtype
 from jal.db.db import JalDB
+from jal.db.operations import LedgerTransaction, Dividend
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -16,6 +16,7 @@ class DividendWidgetDelegate(WidgetMapperDelegateBase):
         WidgetMapperDelegateBase.__init__(self, parent)
         self.delegates = {'timestamp': self.timestamp_delegate,
                           'ex_date': self.timestamp_delegate,
+                          'asset_id': self.symbol_delegate,
                           'amount': self.float_delegate,
                           'tax': self.float_delegate}
 
@@ -25,7 +26,7 @@ class DividendWidget(AbstractOperationDetails):
     def __init__(self, parent=None):
         AbstractOperationDetails.__init__(self, parent)
         self.name = "Dividend"
-        self.operation_type = TransactionType.Dividend
+        self.operation_type = LedgerTransaction.Dividend
         self.combo_model = None
 
         self.date_label = QLabel(self)
@@ -140,13 +141,14 @@ class DividendWidget(AbstractOperationDetails):
 
     @Slot()
     def typeChanged(self, dividend_type_id):
-        self.price_label.setVisible(dividend_type_id == DividendSubtype.StockDividend)
-        self.price_edit.setVisible(dividend_type_id == DividendSubtype.StockDividend)
+        self.price_label.setVisible(dividend_type_id == Dividend.StockDividend)
+        self.price_edit.setVisible(dividend_type_id == Dividend.StockDividend)
         self.refreshAssetPrice()
 
     def refreshAssetPrice(self):
-        if self.type.currentIndex() == DividendSubtype.StockDividend:
+        if self.type.currentIndex() == Dividend.StockDividend:
             price = JalDB().get_quote(self.asset_widget.selected_id,
+                                      JalDB().get_account_currency(self.account_widget.selected_id),
                                       self.timestamp_editor.dateTime().toSecsSinceEpoch())
             if price is not None:
                 self.price_edit.setText(str(price))
