@@ -1,11 +1,9 @@
-import logging
 from functools import partial
 
 from PySide6.QtCore import Qt, Signal, Property, Slot
 from PySide6.QtWidgets import QDialog, QMessageBox, QMenu, QWidgetAction, QLabel
 
 from jal.ui.ui_reference_data_dlg import Ui_ReferenceDataDialog
-from jal.widgets.helpers import decodeError
 from jal.db.helpers import load_icon
 
 
@@ -19,6 +17,7 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
         self.setupUi(self)
 
         self.model = None
+        self._previous_row = -1
         self.selected_id = 0
         self.p_selected_name = ''
         self._filter_text = ''
@@ -27,11 +26,15 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
         self.group_key_field = None
         self.group_key_index = None
         self.group_fkey_field = None
+        self.filter_field = None
+        self._filter_value = ''
         self.toggle_state = False
         self.toggle_field = None
         self.search_field = None
         self.search_text = ""
         self.tree_view = False
+        self.toolbar = None
+        self.custom_editor = False
 
         self.AddChildBtn.setVisible(False)
         self.GroupLbl.setVisible(False)
@@ -53,7 +56,9 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
         self.CommitBtn.clicked.connect(self.OnCommit)
         self.RevertBtn.clicked.connect(self.OnRevert)
         self.DataView.doubleClicked.connect(self.OnDoubleClicked)
+        self.DataView.clicked.connect(self.OnClicked)
         self.TreeView.doubleClicked.connect(self.OnDoubleClicked)
+        self.TreeView.clicked.connect(self.OnClicked)
 
     def _init_completed(self):
         self.DataView.setVisible(not self.tree_view)
@@ -134,14 +139,20 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
 
     @Slot()
     def OnAdd(self):
-        if self.tree_view:
-            idx = self.TreeView.selectionModel().selection().indexes()
+        if self.custom_editor:
+            editor = self.customEditor()
+            editor.createNewRecord()
+            editor.exec()
+            self.model.select()   # TODO better to make self.beginInsertRows/endInsertRows
         else:
-            idx = self.DataView.selectionModel().selection().indexes()
-        current_index = idx[0] if idx else self.model.index(0, 0)
-        self.model.addElement(current_index, in_group=self.group_id)
-        self.CommitBtn.setEnabled(True)
-        self.RevertBtn.setEnabled(True)
+            if self.tree_view:
+                idx = self.TreeView.selectionModel().selection().indexes()
+            else:
+                idx = self.DataView.selectionModel().selection().indexes()
+            current_index = idx[0] if idx else self.model.index(0, 0)
+            self.model.addElement(current_index, in_group=self.group_id)
+            self.CommitBtn.setEnabled(True)
+            self.RevertBtn.setEnabled(True)
 
     @Slot()
     def OnChildAdd(self):
@@ -166,15 +177,6 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
     @Slot()
     def OnCommit(self):
         if not self.model.submitAll():
-            if self.model.lastError().nativeErrorCode() == '1299':
-                prefix = "NOT NULL constraint failed: " + self.model.tableName() + "."
-                if self.model.lastError().databaseText().startswith(prefix):
-                    field_name = self.model.lastError().databaseText()[len(prefix):]
-                    header_title = self.model.headerData(self.model.fieldIndex(field_name))
-                    QMessageBox().warning(self, self.tr("Data are incomplete"),
-                                          self.tr("Column has no valid value: " + header_title), QMessageBox.Ok)
-                    return
-            logging.fatal(self.tr("Submit failed: ") + decodeError(self.model.lastError().text()))
             return
         self.CommitBtn.setEnabled(False)
         self.RevertBtn.setEnabled(False)
@@ -185,6 +187,12 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
         self.CommitBtn.setEnabled(False)
         self.RevertBtn.setEnabled(False)
 
+    def setFilterValue(self, filter_value):
+        if self.filter_field is None:
+            return
+        self._filter_value = filter_value
+        self.setFilter()
+
     def resetFilter(self):
         self.model.setFilter("")
 
@@ -202,9 +210,15 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
                                   f"WHERE {search[3]} LIKE '%{self.search_text}%')")
             else:
                 assert False, f"Unsupported format of search field: {self.search_field}"
+
         if self.group_id:
             conditions.append(f"{self.table}.{self.group_key_field}={self.group_id}")
 
+        if self.filter_field is not None and self._filter_value:
+            conditions.append(f"{self.table}.{self.filter_field} = {self._filter_value}")
+            # completion model needs only this filter, others are for dialog
+            self.model.completion_model.setFilter(f"{self.table}.{self.filter_field} = {self._filter_value}")
+
         if self.toggle_field:
             if not self.toggle_state:
                 conditions.append(f"{self.table}.{self.toggle_field}=1")
@@ -228,6 +242,16 @@ class ReferenceDataDialog(QDialog, Ui_ReferenceDataDialog):
             self.selected_id = self.model.getId(idx[0])
             self.p_selected_name = self.model.getName(idx[0])
 
+    @Slot()
+    def OnClicked(self, index):
+        if self.custom_editor:
+            if self._previous_row == index.row():
+                editor = self.customEditor()
+                editor.selected_id = self.selected_id
+                editor.exec()
+            else:
+                self._previous_row = index.row()
+
     @Slot()
     def OnDoubleClicked(self, index):
         self.selected_id = self.model.getId(index)
