@@ -52,6 +52,9 @@ class AbstractReferenceSelector(QWidget):
 
     selected_id = Property(int, getId, setId, notify=changed, user=True)
 
+    def setFilterValue(self, filter_value):
+        self.dialog.setFilterValue(filter_value)
+
     def on_button_clicked(self):
         ref_point = self.mapToGlobal(self.name.geometry().bottomLeft())
         self.dialog.setGeometry(ref_point.x(), ref_point.y(), self.dialog.width(), self.dialog.height())
@@ -69,6 +72,7 @@ class AbstractReferenceSelector(QWidget):
     def isCustom(self):
         return True
 
+
 # ----------------------------------------------------------------------------------------------------------------------
 class AccountSelector(AbstractReferenceSelector):
     def __init__(self, parent=None):
@@ -78,14 +82,16 @@ class AccountSelector(AbstractReferenceSelector):
         self.dialog = ui_dialogs.AccountListDialog()
         AbstractReferenceSelector.__init__(self, parent)
 
+
 class AssetSelector(AbstractReferenceSelector):
     def __init__(self, parent=None):
-        self.table = "assets"
-        self.selector_field = "name"
+        self.table = "assets_ext"
+        self.selector_field = "symbol"
         self.details_field = "full_name"
         self.dialog = ui_dialogs.AssetListDialog()
         AbstractReferenceSelector.__init__(self, parent)
 
+
 class PeerSelector(AbstractReferenceSelector):
     def __init__(self, parent=None):
         self.table = "agents"
@@ -94,6 +100,7 @@ class PeerSelector(AbstractReferenceSelector):
         self.dialog = ui_dialogs.PeerListDialog()
         AbstractReferenceSelector.__init__(self, parent)
 
+
 class CategorySelector(AbstractReferenceSelector):
     def __init__(self, parent=None):
         self.table = "categories"
@@ -102,6 +109,7 @@ class CategorySelector(AbstractReferenceSelector):
         self.dialog = ui_dialogs.CategoryListDialog()
         AbstractReferenceSelector.__init__(self, parent)
 
+
 class TagSelector(AbstractReferenceSelector):
     def __init__(self, parent=None):
         self.table = "tags"
